@@ -24,9 +24,12 @@
 #include "Time.h"
 
 #if defined(WIN32)
- #include <windows.h>
+ #include <mednafen/win32-common.h>
 #endif
 
+namespace Mednafen
+{
+
 namespace Time
 {
 
@@ -238,3 +241,5 @@ void SleepMS(uint32 amount) noexcept
 }
 
 }
+
+}
