@@ -74,5 +74,5 @@ bool GT_ReinitSound(void);
 void BuildSystemSetting(MDFNSetting *setting, const char *system_name, const char *name, const char *description, const char *description_extra, MDFNSettingType type, 
 	const char *default_value, const char *minimum = NULL, const char *maximum = NULL,
 	bool (*validate_func)(const char *name, const char *value) = NULL, void (*ChangeNotification)(const char *name) = NULL, 
-        const MDFNSetting_EnumList *enum_list = NULL);
+        const MDFNSetting_EnumList *enum_list = NULL, uint32 extra_flags = 0);
 #endif
