@@ -2,7 +2,7 @@
 /* Mednafen - Multi-system Emulator                                           */
 /******************************************************************************/
 /* memdebugger.cpp:
-**  Copyright (C) 2007-2016 Mednafen Team
+**  Copyright (C) 2007-2018 Mednafen Team
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -223,7 +223,7 @@ void MemDebugger::PromptFinish(const std::string &pstring)
 	 {
 	  if(ICV_Init(pstring.c_str()))
 	  {
-	   MDFNI_SetSetting(std::string(CurGame->shortname) + "." + std::string("debugger.memcharenc"), pstring);
+	   MDFNI_SetSetting(std::string(CurGame->shortname) + "." + "debugger.memcharenc", pstring);
 	  }
 	 }
          else if(which == DumpMem)
@@ -261,6 +261,8 @@ void MemDebugger::PromptFinish(const std::string &pstring)
 	    fp.write(write_buffer, to_write);
 	    a += to_write;
 	   }
+
+           fp.close();
           }
          }
          else if(which == LoadMem)
@@ -767,13 +769,17 @@ int MemDebugger::Event(const SDL_Event *event)
 
 	if(event->key.keysym.mod & KMOD_LALT)
 	 break;
+	//
+	//
+	//
+	const unsigned ks = event->key.keysym.sym;
 
         if(InPrompt)
         {
          myprompt->Event(event);
         }
-	else if(InEditMode && !InTextArea && ((event->key.keysym.sym >= SDLK_0 && event->key.keysym.sym <= SDLK_9)	|| 
-	   (event->key.keysym.sym >= SDLK_a && event->key.keysym.sym <= SDLK_f)))
+	else if(InEditMode && !InTextArea && ((ks >= SDLK_0 && ks <= SDLK_9)	|| 
+	   (ks >= SDLK_a && ks <= SDLK_f)))
 	{
          uint8 tc = 0;
          uint8 meowbyte = 0;
@@ -793,7 +799,12 @@ int MemDebugger::Event(const SDL_Event *event)
          LowNib = !LowNib;
          if(!LowNib)
 	  ChangePos(1);
-        }
+	}
+	else if(InEditMode && InTextArea && ks != SDLK_TAB && ks != SDLK_RETURN && ks != SDLK_INSERT && ks != SDLK_END && ks != SDLK_HOME &&
+		ks != SDLK_PAGEUP && ks != SDLK_PAGEDOWN && ks != SDLK_UP && ks != SDLK_DOWN && ks != SDLK_LEFT && ks != SDLK_RIGHT)
+	{
+	 //
+	}
 	else switch(event->key.keysym.sym)
 	{
 	 default: break;
@@ -978,7 +989,7 @@ MemDebugger::MemDebugger() : AddressSpaces(NULL), ASpace(NULL), IsActive(false),
    SizeCache[i] = tmpsize;
   }
 
-  ICV_Init( MDFN_GetSettingS(std::string(CurGame->shortname) + "." + std::string("debugger.memcharenc")).c_str() );
+  ICV_Init( MDFN_GetSettingS(std::string(CurGame->shortname) + "." + "debugger.memcharenc").c_str() );
  }
 }
 
