@@ -11,7 +11,10 @@ int Netplay_GetTextView(void);
 bool Netplay_IsTextInput(void);
 bool Netplay_TryTextExit(void);
 
-extern int MDFNDnetplay;
+// Returns local player mask if netplay is active, ~(uint32)0 otherwise
+uint32 Netplay_GetLPM(void);
+
+MDFN_HIDE extern int MDFNDnetplay;
 
 
 //
