@@ -24,6 +24,9 @@
 
 #include <mednafen/mednafen.h>
 
+namespace Mednafen
+{
+
 class M68K
 {
  public:
@@ -77,7 +80,10 @@ class M68K
   XPENDING_MASK_NMI	= 0x0002,
   XPENDING_MASK_RESET	= 0x0010,
   XPENDING_MASK_STOPPED	= 0x0100,	// via STOP instruction
-  XPENDING_MASK_EXTHALTED= 0x1000
+  XPENDING_MASK_EXTHALTED= 0x1000,
+
+  // For save state sanitizing:
+  XPENDING_MASK__VALID = XPENDING_MASK_INT | XPENDING_MASK_NMI | XPENDING_MASK_RESET | XPENDING_MASK_STOPPED | XPENDING_MASK_EXTHALTED
  };
 
  const bool Revision_E;
@@ -493,4 +499,5 @@ static uint8 TAS_Callback(uint8 data)
  }
 };
 
+}
 #endif
