@@ -19,12 +19,11 @@
  */
 
 #include <mednafen/mednafen.h>
+#include <trio/trio.h>
 #include "pce_psg.h"
 
-#include <math.h>
-#include <string.h>
-#include <trio/trio.h>
-
+namespace Mednafen
+{
 // Frequency cache cutoff optimization threshold (<= FREQC7M_COT)
 #define FREQC7M_COT	0x7 //0xA
 
@@ -908,3 +907,5 @@ void PCE_PSG::StateAction(StateMem *sm, const unsigned load, const bool data_onl
   }
  }
 }
+
+}
