@@ -19,7 +19,9 @@
 #include <mednafen/mednafen.h>
 #include <mednafen/mempatcher.h>
 
-#include "TLCS-900h/TLCS900h_disassemble.h"
+using namespace Mednafen;
+
+//#include "TLCS-900h/TLCS900h_disassemble.h"
 #include "TLCS-900h/TLCS900h_interpret_dst.h"
 #include "TLCS-900h/TLCS900h_interpret.h"
 #include "TLCS-900h/TLCS900h_interpret_reg.h"
