@@ -13,7 +13,7 @@ typedef enum
  SYSCARD_ARCADE		// 3.0 + extras
 } SysCardType;
 
-uint32 HuC_Load(MDFNFILE* fp, bool DisableBRAM = false, SysCardType syscard = SYSCARD_NONE) MDFN_COLD;
+uint32 HuC_Load(Stream* s, bool DisableBRAM = false, SysCardType syscard = SYSCARD_NONE) MDFN_COLD;
 void HuC_SaveNV(void);
 void HuC_Kill(void) MDFN_COLD;
 
