@@ -399,7 +399,7 @@ void PS_SPU::RunDecoder(SPU_Voice *voice)
     }
     else
     {
-     if(voice->LoopAddr != voice->CurAddr)
+     if((voice->LoopAddr ^ voice->CurAddr) & ~0x7)
      {
       PSX_DBG(PSX_DBG_FLOOD, "[SPU] Ignore: LoopAddr=0x%08x, SampLA=0x%08x\n", voice->LoopAddr, voice->CurAddr);
      }
@@ -685,6 +685,11 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
 
    //PSX_WARNING("[SPU] Voice %d CurPhase=%08x, pitch=%04x, CurAddr=%08x", voice_num, voice->CurPhase, voice->Pitch, voice->CurAddr);
 
+   if(voice->DecodePlayDelay)
+   {
+    voice->IgnoreSampLA = false;
+   }
+
    //
    // Decode new samples if necessary.
    //
@@ -772,7 +777,26 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
    {
     if(voice->ADSR.Phase != ADSR_RELEASE)
     {
-     ReleaseEnvelope(voice);
+     // TODO/FIXME:
+     //  To fix all the missing notes in "Dragon Ball GT: Final Bout" music, !voice->DecodePlayDelay instead of
+     //  voice->DecodePlayDelay < 3 is necessary, but that would cause the length of time for which the voice off is
+     //  effectively ignored to be too long by about half a sample(rough test measurement).  That, combined with current
+     //  CPU and DMA emulation timing inaccuracies(execution generally too fast), creates a significant risk of regressions
+     //  in other games, so be very conservative for now.
+     //
+     //  Also, voice on should be ignored during the delay as well, but comprehensive tests are needed before implementing that
+     //  due to some effects that appear to occur repeatedly during the delay on a PS1 but are currently only emulated as
+     //  performed when the voice on is processed(e.g. curaddr = startaddr).
+     //
+     if(voice->DecodePlayDelay < 3)
+     {
+      if(voice->DecodePlayDelay)
+       PSX_DBG(PSX_DBG_WARNING, "[SPU] Voice %u off maybe should be ignored, but isn't due to current emulation code limitations; dpd=%u\n", voice_num, voice->DecodePlayDelay);
+
+      ReleaseEnvelope(voice);
+     }
+     else
+      PSX_DBG(PSX_DBG_WARNING, "[SPU] Voice %u off ignored.\n", voice_num);
     }
    }
 
@@ -969,6 +993,11 @@ void PS_SPU::Write(pscpu_timestamp_t timestamp, uint32 A, uint16 V)
 
    case 0x0E: voice->LoopAddr = (V << 2) & 0x3FFFF;
 	      voice->IgnoreSampLA = true;
+
+	      if(voice->DecodePlayDelay || (VoiceOn & (1U << (voice - Voices))))
+	      {
+	       PSX_WARNING("[SPU] Loop address for voice %u written during voice on delay.", (unsigned)(voice - Voices));
+	      }
 	      //if((voice - Voices) == 22)
 	      //{
 	      // SPUIRQ_DBG("Manual loop address setting for voice %d: %04x", (int)(voice - Voices), V);
@@ -1077,16 +1106,16 @@ void PS_SPU::Write(pscpu_timestamp_t timestamp, uint32 A, uint16 V)
    case 0x2C: PSX_WARNING("[SPU] Global reg 0x2c set: 0x%04x", V);
 	      break;
 
-   case 0x30: CDVol[0] = V;
+   case 0x30: CDVol[0] = (int16)V;
 	      break;
 
-   case 0x32: CDVol[1] = V;
+   case 0x32: CDVol[1] = (int16)V;
 	      break;
 
-   case 0x34: ExternVol[0] = V;
+   case 0x34: ExternVol[0] = (int16)V;
 	      break;
 
-   case 0x36: ExternVol[1] = V;
+   case 0x36: ExternVol[1] = (int16)V;
 	      break;
 
    case 0x38:
