@@ -2,7 +2,7 @@
 /* Mednafen Fast SNES Emulation Module                                        */
 /******************************************************************************/
 /* snes.cpp:
-**  Copyright (C) 2015-2017 Mednafen Team
+**  Copyright (C) 2015-2019 Mednafen Team
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -22,6 +22,7 @@
 #include "snes.h"
 #include "input.h"
 #include "cart.h"
+#include "msu1.h"
 #include "apu.h"
 #include "ppu.h"
 
@@ -30,6 +31,7 @@
 #include <mednafen/SNSFLoader.h>
 #include <mednafen/player.h>
 #include <mednafen/hash/sha1.h>
+#include <mednafen/cheat_formats/snes.h>
 
 #include <bitset>
 
@@ -41,20 +43,48 @@ static bool SpecEx, SpecExSoundToo;
 static MemoryStream* SpecExSS = NULL;
 static int32 SpecExAudioExpected;
 
-static void EventReset(void);
+struct ReadPatchInfoStruct
+{
+ bool operator<(const ReadPatchInfoStruct& o) const
+ {
+  return address < o.address;
+ }
+ ReadPatchInfoStruct() { }
+ ReadPatchInfoStruct(const uint32 a) : address(a) { }
+
+ uint32 address;
+ uint8 value;
+ int compare;
+ uint8 prev_mapping;
+};
+static std::vector<ReadPatchInfoStruct> ReadPatchInfo;
+
+static void InitEvents(void) MDFN_COLD;
 
 
 template<bool b_bus>
 static void SetHandlers(uint32 A1, uint32 A2, readfunc read_handler, writefunc write_handler)
 {
+ // entry 0xFF is reserved for game genie cheats handler
  unsigned index = 0;
 
- assert(read_handler && write_handler);
+ if(A1 != A2 || b_bus)
+ {
+  assert(read_handler && write_handler);
+ }
+ else
+ {
+  if(!read_handler)
+   read_handler = CPUM.ReadFuncs[CPUM.RWIndex[A1]];
 
- while(index < 256 && CPUM.ReadFuncs[index] && CPUM.WriteFuncs[index] && (CPUM.ReadFuncs[index] != read_handler || CPUM.WriteFuncs[index] != write_handler))
+  if(!write_handler)
+   write_handler = CPUM.WriteFuncs[CPUM.RWIndex[A1]];
+ }
+
+ while(index < 255 && CPUM.ReadFuncs[index] && CPUM.WriteFuncs[index] && (CPUM.ReadFuncs[index] != read_handler || CPUM.WriteFuncs[index] != write_handler))
   index++;
 
- assert(index < 256);
+ assert(index < 255);
 
  CPUM.ReadFuncs[index] = read_handler;
  CPUM.WriteFuncs[index] = write_handler;
@@ -100,15 +130,18 @@ void Set_B_Handlers(uint8 A1, uint8 A2, readfunc read_handler, writefunc write_h
  SetHandlers<true>(A1, A2, read_handler, write_handler);
 }
 
-bool MemSelect;
-
 template<signed cyc>
 static INLINE DEFREAD(OBRead)
 {
+ if(MDFN_UNLIKELY(DBG_InHLRead))
+ {
+  return CPUM.mdr;
+ }
+
  if(cyc >= 0)
   CPUM.timestamp += cyc;
  else
-  CPUM.timestamp += MemSelect ? MEMCYC_FAST : MEMCYC_SLOW;
+  CPUM.timestamp += CPUM.MemSelectCycles;
 
  SNES_DBG("[SNES] Unknown Read: $%02x:%04x\n", A >> 16, A & 0xFFFF);
  return CPUM.mdr;
@@ -120,7 +153,7 @@ static INLINE DEFWRITE(OBWrite)
  if(cyc >= 0)
   CPUM.timestamp += cyc;
  else
-  CPUM.timestamp += MemSelect ? MEMCYC_FAST : MEMCYC_SLOW;
+  CPUM.timestamp += CPUM.MemSelectCycles;
 
  SNES_DBG("[SNES] Unknown Write: $%02x:%04x $%02x\n", A >> 16, A & 0xFFFF, V);
 }
@@ -144,11 +177,16 @@ static std::bitset<0x20000> WRAMWritten;	// for debugging
 template<uint32 mask>
 static DEFREAD(WRAMRead)
 {
+ if(MDFN_UNLIKELY(DBG_InHLRead))
+ {
+  return WRAM[A & mask];
+ }
+
  CPUM.timestamp += MEMCYC_SLOW;
 
 #ifdef SNES_DBG_ENABLE
- if(!WRAMWritten[A & mask])
-  SNES_DBG("[SNES] Read from uninitialized WRAM at 0x%08x!\n", A & mask);
+ //if(!WRAMWritten[A & mask])
+ // SNES_DBG("[SNES] Read from uninitialized WRAM at 0x%08x!\n", A & mask);
 #endif
 
  return WRAM[A & mask];
@@ -189,7 +227,7 @@ static void ICRegsReset(bool powering_up)
 
  WMAddress = 0;
 
- MemSelect = 0;
+ CPUM.MemSelectCycles = MEMCYC_SLOW;
 }
 
 static DEFWRITE(ICRegsWrite)
@@ -226,7 +264,7 @@ static DEFWRITE(ICRegsWrite)
 		}
 		break;
 
-  case 0x420D:  MemSelect = V & 1;
+  case 0x420D:  CPUM.MemSelectCycles = (V & 1) ? MEMCYC_FAST : MEMCYC_SLOW;;
 		break;
  }
 }
@@ -234,6 +272,11 @@ static DEFWRITE(ICRegsWrite)
 
 static DEFREAD(Read_4214)
 {
+ if(MDFN_UNLIKELY(DBG_InHLRead))
+ {
+  return DivQuotient >> 0;
+ }
+
  CPUM.timestamp += MEMCYC_FAST;
 
  return DivQuotient >> 0;
@@ -241,6 +284,11 @@ static DEFREAD(Read_4214)
 
 static DEFREAD(Read_4215)
 {
+ if(MDFN_UNLIKELY(DBG_InHLRead))
+ {
+  return DivQuotient >> 8;
+ }
+
  CPUM.timestamp += MEMCYC_FAST;
 
  return DivQuotient >> 8;
@@ -248,6 +296,11 @@ static DEFREAD(Read_4215)
 
 static DEFREAD(Read_4216)
 {
+ if(MDFN_UNLIKELY(DBG_InHLRead))
+ {
+  return MultProduct >> 0;
+ }
+
  CPUM.timestamp += MEMCYC_FAST;
 
  return MultProduct >> 0;
@@ -255,6 +308,11 @@ static DEFREAD(Read_4216)
 
 static DEFREAD(Read_4217)
 {
+ if(MDFN_UNLIKELY(DBG_InHLRead))
+ {
+  return MultProduct >> 8;
+ }
+
  CPUM.timestamp += MEMCYC_FAST;
 
  return MultProduct >> 8;
@@ -301,6 +359,11 @@ static DEFWRITE(Write_2183)
 
 static DEFREAD(Read_2180)
 {
+ if(MDFN_UNLIKELY(DBG_InHLRead))
+ {
+  return WRAM[WMAddress];
+ }
+
  uint8 ret;
 
  CPUM.timestamp += MEMCYC_FAST;
@@ -311,6 +374,36 @@ static DEFREAD(Read_2180)
  return ret;
 }
 
+static INLINE ReadPatchInfoStruct FindRPie(uint32 A)
+{
+ A &= 0xFFFFFF;
+ //
+ std::vector<ReadPatchInfoStruct>::const_iterator it = std::lower_bound(ReadPatchInfo.begin(), ReadPatchInfo.end(), A);
+
+ assert(it != ReadPatchInfo.end() && it->address == A);
+
+ return *it;
+}
+
+template<bool ForceA>
+static DEFREAD_NOHOT(Read_CheatHook)
+{
+ ReadPatchInfoStruct rpie = FindRPie(A);
+ uint8 ret = (ForceA ? CPUM.ReadFuncsA : CPUM.ReadFuncs)[rpie.prev_mapping](A);
+
+ if(rpie.compare < 0 || ret == rpie.compare)
+  ret = rpie.value;
+
+ return ret;
+}
+
+template<bool ForceA>
+static DEFWRITE_NOHOT(Write_CheatHook)
+{
+ ReadPatchInfoStruct rpie = FindRPie(A);
+ (ForceA ? CPUM.WriteFuncsA : CPUM.WriteFuncs)[rpie.prev_mapping](A, V);
+}
+
 //
 //
 //
@@ -345,12 +438,13 @@ static MDFN_COLD void Reset(bool powering_up)
  APU_Reset(powering_up);
  PPU_Reset(powering_up);
  CART_Reset(powering_up);
+ MSU1_Reset(powering_up);
  INPUT_Reset(powering_up);
 
  //
  CPU_Reset(powering_up);
 
- EventReset();
+ InitEvents();
  ForceEventUpdates(0);
 }
 
@@ -374,6 +468,8 @@ static MDFN_COLD bool TestMagic(GameFile* gf)
 
 static MDFN_COLD void Cleanup(void)
 {
+ ReadPatchInfo.clear();
+
  if(SpecExSS)
  {
   delete SpecExSS;
@@ -398,10 +494,22 @@ static MDFN_COLD void Cleanup(void)
   PPU_Kill();
   INPUT_Kill();
   CART_Kill();
+  MSU1_Kill();
  }
 }
 
-static MDFN_COLD void Load(GameFile* gf)
+enum
+{
+ REGION_AUTO = 0,
+ REGION_NTSC,
+ REGION_PAL,
+ REGION_NTSC_LIE_AUTO,
+ REGION_PAL_LIE_AUTO,
+ REGION_NTSC_LIE_PAL,
+ REGION_PAL_LIE_NTSC
+};
+
+static MDFN_COLD void LoadReal(GameFile* gf)
 {
  if(SPCReader::TestMagic(gf->stream))
  {
@@ -410,7 +518,7 @@ static MDFN_COLD void Load(GameFile* gf)
   EmulatedSNES_Faust.fps = (1U << 24) * 75;
   EmulatedSNES_Faust.MasterClock = MDFN_MASTERCLOCK_FIXED(21477272.7);
 
-  APU_Init(false);
+  EmulatedSNES_Faust.IdealSoundRate = APU_Init(false, (double)EmulatedSNES_Faust.MasterClock / MDFN_MASTERCLOCK_FIXED(1));
   Reset(true);
 
   return;
@@ -420,10 +528,15 @@ static MDFN_COLD void Load(GameFile* gf)
  SpecExSoundToo = MDFN_GetSettingB("snes_faust.spex.sound");
  SpecExAudioExpected = -1;
 
+ MDFN_printf("SpecEx: %u\n", SpecEx);
+ MDFN_printf("SpecExSoundToo: %u\n", SpecExSoundToo);
+
  MDFNMP_Init(8192, (1U << 24) / 8192);
  MDFNMP_RegSearchable(0x7E0000, 0x20000);
 
- CPU_Init();
+ DBG_Init();
+
+ CPU_Init(&CPUM);
 
  memset(CPUM.ReadFuncs, 0, sizeof(CPUM.ReadFuncs));
  memset(CPUM.WriteFuncs, 0, sizeof(CPUM.WriteFuncs));
@@ -431,6 +544,12 @@ static MDFN_COLD void Load(GameFile* gf)
  memset(CPUM.ReadFuncsA, 0, sizeof(CPUM.ReadFuncsA));
  memset(CPUM.WriteFuncsA, 0, sizeof(CPUM.WriteFuncsA));
 
+ CPUM.ReadFuncs[0xFF] = Read_CheatHook<false>;
+ CPUM.WriteFuncs[0xFF] = Write_CheatHook<false>;
+
+ CPUM.ReadFuncsA[0xFF] = Read_CheatHook<true>;
+ CPUM.WriteFuncsA[0xFF] = Write_CheatHook<true>;
+
  //
  // Map in open bus.
  //
@@ -485,16 +604,73 @@ static MDFN_COLD void Load(GameFile* gf)
   }
  }
 
- if(SNSFLoader::TestMagic(gf->stream))
- {
-  snsf_loader = new SNSFLoader(gf->vfs, gf->dir, gf->stream);
-  Player_Init(1, snsf_loader->tags.GetTag("game"), snsf_loader->tags.GetTag("artist"), snsf_loader->tags.GetTag("copyright"), std::vector<std::string>({ snsf_loader->tags.GetTag("title") }));
- }
-
- const bool IsPAL = CART_Init(snsf_loader ? &snsf_loader->ROM_Data : gf->stream, EmulatedSNES_Faust.MD5);
- CART_LoadNV();
-
  DMA_Init();
+ //
+ //
+ if(SNSFLoader::TestMagic(gf->stream))
+  snsf_loader = new SNSFLoader(gf->vfs, gf->dir, gf->stream);
+
+ const int32 cx4_ocmultiplier = ((MDFN_GetSettingUI("snes_faust.cx4.clock_rate") << 16) + 50) / 100;
+ const int32 superfx_ocmultiplier = ((MDFN_GetSettingUI("snes_faust.superfx.clock_rate") << 16) + 50) / 100;
+ const bool superfx_enable_icache = MDFN_GetSettingB("snes_faust.superfx.icache");
+ const bool CartIsPAL = CART_Init(snsf_loader ? &snsf_loader->ROM_Data : gf->stream, EmulatedSNES_Faust.MD5, cx4_ocmultiplier, superfx_ocmultiplier, superfx_enable_icache);
+ const unsigned region = MDFN_GetSettingUI("snes_faust.region");
+ bool IsPAL, IsPALPPUBit;
+
+ if(snsf_loader)
+ {
+  uint8* const cart_ram = CART_GetRAMPointer();
+
+  if(cart_ram)
+  {
+   const size_t cart_ram_size = CART_GetRAMSize();
+   const size_t read_size = std::min<size_t>(cart_ram_size, snsf_loader->SRAM_Data.size());
+
+   memset(cart_ram, 0xFF, cart_ram_size);
+   snsf_loader->SRAM_Data.read(cart_ram, read_size);
+  }
+ }
+ else
+  CART_LoadNV();
+
+ switch(region)
+ {
+  default:
+	assert(0);
+  case REGION_AUTO:
+	IsPAL = IsPALPPUBit = CartIsPAL;
+	break;
+
+  case REGION_NTSC:
+	IsPAL = IsPALPPUBit = false;
+	break;
+
+  case REGION_PAL:
+	IsPAL = IsPALPPUBit = true;
+	break;
+
+  case REGION_NTSC_LIE_AUTO:
+	IsPAL = false;
+	IsPALPPUBit = CartIsPAL;
+	break;
+
+  case REGION_PAL_LIE_AUTO:
+	IsPAL = true;
+	IsPALPPUBit = CartIsPAL;
+	break;
+
+  case REGION_NTSC_LIE_PAL:
+	IsPAL = false;
+	IsPALPPUBit = true;
+	break;
+
+  case REGION_PAL_LIE_NTSC:
+	IsPAL = true;
+	IsPALPPUBit = false;
+	break;
+ }
+ //
+ //
  INPUT_Init();
  {
   const bool mte[2] = { MDFN_GetSettingB("snes_faust.input.sport1.multitap"), MDFN_GetSettingB("snes_faust.input.sport2.multitap") };
@@ -502,15 +678,43 @@ static MDFN_COLD void Load(GameFile* gf)
  }
 
  EmulatedSNES_Faust.MasterClock = IsPAL ? MDFN_MASTERCLOCK_FIXED(21281370.0) : MDFN_MASTERCLOCK_FIXED(21477272.7);
+ EmulatedSNES_Faust.VideoSystem = IsPAL ? VIDSYS_PAL : VIDSYS_NTSC;
 
- PPU_Init(IsPAL);
- PPU_SetGetVideoParams(&EmulatedSNES_Faust, MDFN_GetSettingB("snes_faust.correct_aspect"));
+ PPU_Init(MDFN_GetSettingUI("snes_faust.renderer"), IsPAL, IsPALPPUBit, MDFN_GetSettingB("snes_faust.frame_begin_vblank"), MDFN_GetSettingUI("snes_faust.affinity.ppu"));
+ //
+ unsigned sls = MDFN_GetSettingUI(IsPAL ? "snes_faust.slstartp" : "snes_faust.slstart");
+ unsigned sle = MDFN_GetSettingUI(IsPAL ? "snes_faust.slendp" : "snes_faust.slend");
 
- APU_Init(IsPAL);
+ if(sle < sls)
+  std::swap(sls, sle);
 
+ PPU_SetGetVideoParams(&EmulatedSNES_Faust, MDFN_GetSettingUI("snes_faust.correct_aspect"), MDFN_GetSettingUI("snes_faust.h_filter"), sls, sle);
+
+ EmulatedSNES_Faust.IdealSoundRate = APU_Init(IsPAL, (double)EmulatedSNES_Faust.MasterClock / MDFN_MASTERCLOCK_FIXED(1));
+ MSU1_Init(gf, &EmulatedSNES_Faust.IdealSoundRate, MDFN_GetSettingUI("snes_faust.affinity.msu1.audio"), MDFN_GetSettingUI("snes_faust.affinity.msu1.data"));
+ //
+ if(snsf_loader)
+  Player_Init(1, snsf_loader->tags.GetTag("game"), snsf_loader->tags.GetTag("artist"), snsf_loader->tags.GetTag("copyright"), std::vector<std::string>({ snsf_loader->tags.GetTag("title") }));
+ //
+ //
+ //
  Reset(true);
 }
 
+static MDFN_COLD void Load(GameFile* gf)
+{
+ try
+ {
+  LoadReal(gf);
+ }
+ catch(...)
+ {
+  Cleanup();
+
+  throw;
+ }
+}
+
 static MDFN_COLD void CloseGame(void)
 {
  if(!snsf_loader && !spc_reader)
@@ -563,22 +767,56 @@ static MDFN_COLD void CheatMemWrite(uint32 A, uint8 V)
   *p = V;
 }
 
-struct event_list_entry
+static MDFN_COLD void CheatInstallReadPatch(uint32 address, uint8 value, int compare)
 {
- uint32 which;
- uint32 event_time;
- event_list_entry *prev;
- event_list_entry *next;
-};
+ //printf("ReadPatch: %08x %d %d\n", address, value, compare);
+ address &= 0xFFFFFF;
+ //
+ {
+  std::vector<ReadPatchInfoStruct>::iterator it = std::lower_bound(ReadPatchInfo.begin(), ReadPatchInfo.end(), address);
 
-static event_list_entry events[SNES_EVENT__COUNT];
+  if(it != ReadPatchInfo.end() && it->address == address)
+  {
+   it->value = value;
+   it->compare = compare;
+  }
+  else
+  {
+   assert(it == ReadPatchInfo.end() || it->address > address);
+   //
+   ReadPatchInfoStruct rpi;
 
-static void EventReset(void)
+   rpi.address = address;
+   rpi.value = value;
+   rpi.compare = compare;
+   rpi.prev_mapping = CPUM.RWIndex[address];
+   assert(rpi.prev_mapping != 0xFF);
+
+   ReadPatchInfo.insert(it, rpi);
+  }
+ }
+ //
+ CPUM.RWIndex[address] = 0xFF;
+}
+
+static MDFN_COLD void CheatRemoveReadPatches(void)
+{
+ for(auto const& rpie : ReadPatchInfo)
+ {
+  assert(CPUM.RWIndex[rpie.address] == 0xFF);
+  assert(rpie.prev_mapping != 0xFF);
+  CPUM.RWIndex[rpie.address] = rpie.prev_mapping;
+ }
+
+ ReadPatchInfo.clear();
+}
+
+event_list_entry events[SNES_EVENT__COUNT];
+
+static MDFN_COLD void InitEvents(void)
 {
  for(unsigned i = 0; i < SNES_EVENT__COUNT; i++)
  {
-  events[i].which = i;
-
   if(i == SNES_EVENT__SYNFIRST)
    events[i].event_time = 0;
   else if(i == SNES_EVENT__SYNLAST)
@@ -589,31 +827,33 @@ static void EventReset(void)
   events[i].prev = (i > 0) ? &events[i - 1] : NULL;
   events[i].next = (i < (SNES_EVENT__COUNT - 1)) ? &events[i + 1] : NULL;
  }
-}
 
-//static void RemoveEvent(event_list_entry *e)
-//{
-// e->prev->next = e->next;
-// e->next->prev = e->prev;
-//}
+ events[SNES_EVENT_PPU].event_handler = PPU_GetEventHandler();
+ events[SNES_EVENT_PPU_LINEIRQ].event_handler = PPU_GetLineIRQEventHandler();
+ events[SNES_EVENT_DMA_DUMMY].event_handler = DMA_Update;
+ events[SNES_EVENT_CART].event_handler = CART_GetEventHandler();
+ events[SNES_EVENT_MSU1].event_handler = MSU1_GetEventHandler();
+}
 
 static void RebaseTS(const uint32 timestamp)
 {
- for(unsigned i = 0; i < SNES_EVENT__COUNT; i++)
+ for(unsigned i = SNES_EVENT__SYNFIRST + 1; i < SNES_EVENT__SYNLAST; i++)
  {
-  if(i == SNES_EVENT__SYNFIRST || i == SNES_EVENT__SYNLAST)
-   continue;
-
   assert(events[i].event_time > timestamp);
   events[i].event_time -= timestamp;
  }
 
+ CART_AdjustTS(-timestamp);
+ MSU1_AdjustTS(-timestamp);
+
  CPUM.next_event_ts = events[SNES_EVENT__SYNFIRST].next->event_time;
 }
 
 void SNES_SetEventNT(const int type, const uint32 next_timestamp)
 {
+#ifdef SNES_DBG_ENABLE
  assert(type > SNES_EVENT__SYNFIRST && type < SNES_EVENT__SYNLAST);
+#endif
  event_list_entry *e = &events[type];
 
  if(next_timestamp < e->event_time)
@@ -660,15 +900,17 @@ void SNES_SetEventNT(const int type, const uint32 next_timestamp)
   e->event_time = next_timestamp;
  }
 
- CPUM.next_event_ts = events[SNES_EVENT__SYNFIRST].next->event_time; // & Running);
+ CPUM.next_event_ts = events[SNES_EVENT__SYNFIRST].next->event_time & CPUM.running_mask;
 }
 
 // Called from debug.cpp too.
 void ForceEventUpdates(const uint32 timestamp)
 {
- SNES_SetEventNT(SNES_EVENT_PPU, PPU_Update(timestamp));
- SNES_SetEventNT(SNES_EVENT_DMA_DUMMY, DMA_Update(timestamp));
-
+ for(unsigned i = SNES_EVENT__SYNFIRST + 1; i < SNES_EVENT__SYNLAST; i++)
+ {
+  SNES_SetEventNT(i, events[i].event_handler(timestamp));
+ }
+ //
  CPUM.next_event_ts = events[SNES_EVENT__SYNFIRST].next->event_time;
 }
 
@@ -681,24 +923,12 @@ void CPU_Misc::EventHandler(void)
   event_list_entry *prev = e->prev;
   uint32 nt;
 
-  switch(e->which)
-  {
-   default: abort();
-
-   case SNES_EVENT_PPU:
-	nt = PPU_Update(e->event_time);
-	break;
-
-   case SNES_EVENT_DMA_DUMMY:
-	nt = DMA_Update(e->event_time);
-	break;
-
-  }
+  nt = e->event_handler(e->event_time);
 //#if SNES_EVENT_SYSTEM_CHECKS
   assert(nt > e->event_time);
 //#endif
 
-  SNES_SetEventNT(e->which, nt);
+  SNES_SetEventNT(/*e->which*/e - events, nt);
 
   // Order of events can change due to calling SNES_SetEventNT(), this prev business ensures we don't miss an event due to reordering.
   e = prev->next;
@@ -709,7 +939,13 @@ void CPU_Misc::EventHandler(void)
 
 static void NO_INLINE EmulateReal(EmulateSpecStruct* espec)
 {
- APU_StartFrame((double)EmulatedSNES_Faust.MasterClock / MDFN_MASTERCLOCK_FIXED(1), espec->SoundRate);
+ int32 apu_clock_multiplier;
+ int32 resamp_num;
+ int32 resamp_denom;
+ const double master_clock = (double)EmulatedSNES_Faust.MasterClock / MDFN_MASTERCLOCK_FIXED(1);
+ const bool resamp_clear_buf = APU_StartFrame(master_clock, espec->SoundRate, &apu_clock_multiplier, &resamp_num, &resamp_denom);
+
+ MSU1_StartFrame(master_clock, espec->SoundRate, apu_clock_multiplier, resamp_num, resamp_denom, resamp_clear_buf);
 
  if(spc_reader)
   CPUM.timestamp = 286364;
@@ -718,13 +954,54 @@ static void NO_INLINE EmulateReal(EmulateSpecStruct* espec)
   const bool skip_save = espec->skip;
   espec->skip |= snsf_loader != NULL;
 
-  MDFNMP_ApplyPeriodicCheats();
-
-  // Call before PPU_StartFrame(), as PPU_StartFrame() may call INPUT_AutoRead().
-  INPUT_UpdatePhysicalState();
-
   PPU_StartFrame(espec);
 
+#if 0
+ for(unsigned msi = 0; msi < 2; msi++)
+ {
+  for(uint32 A = 0; A < (1U << 24); A++)
+  {
+   CPUM.timestamp = 0;
+   const uint32 pts = CPUM.timestamp;
+
+   CPUM.MemSelectCycles = msi ? MEMCYC_FAST : MEMCYC_SLOW;;
+//   CPUM.ReadA(A);
+   CPUM.WriteA(A, 0);
+   //
+   const uint32 td = CPUM.timestamp - pts;
+   const uint8 bank = A >> 16;
+   const uint16 offs = (uint16)A;
+   uint32 reqtd = ~0U;
+
+   if(!(bank & 0x40))
+   {
+    if(offs < 0x2000)
+     reqtd = 8;
+    else if(offs < 0x4000)
+     reqtd = 6;
+    else if(offs < 0x4200)
+     reqtd = 12;
+    else if(offs < 0x6000)
+     reqtd = 6;
+    else if(offs < 0x8000)
+     reqtd = 8;
+    else
+     reqtd = ((bank & 0x80) && msi) ? 6 : 8;
+   }
+   else
+    reqtd = ((bank & 0x80) && msi) ? 6 : 8;
+
+   if(td != reqtd)
+   {
+    fprintf(stderr, "0x%06x td=%d reqtd=%d\n", A, td, reqtd);
+    assert(td == reqtd);
+   }
+  }
+ }
+ exit(0);
+#endif
+
+
   CPU_Run();
   uint32 prev = CPUM.timestamp;
   ForceEventUpdates(CPUM.timestamp);
@@ -736,6 +1013,7 @@ static void NO_INLINE EmulateReal(EmulateSpecStruct* espec)
  espec->MasterCycles = CPUM.timestamp;
 
  espec->SoundBufSize = APU_EndFrame(espec->SoundBuf);
+ MSU1_EndFrame(espec->SoundBuf, espec->SoundBufSize);
  if(!spc_reader)
  {
   PPU_ResetTS();
@@ -751,7 +1029,11 @@ static sha1_digest doggy;
 static void Emulate(EmulateSpecStruct* espec)
 {
  if(!SpecEx || spc_reader || snsf_loader)
+ {
   EmulateReal(espec);
+  MDFN_MidSync(espec, MIDSYNC_FLAG_NONE);
+  PPU_SyncMT();
+ }
  else
  {
   EmulateSpecStruct tmp_espec = *espec;
@@ -759,7 +1041,8 @@ static void Emulate(EmulateSpecStruct* espec)
   if(espec->SoundFormatChanged || espec->NeedSoundReverse)
    SpecExAudioExpected = -1;
 
-  tmp_espec.skip = true;
+  tmp_espec.skip = -1;
+  assert(tmp_espec.skip == -1);
   tmp_espec.NeedSoundReverse = false;
   tmp_espec.VideoFormatChanged = false;
   tmp_espec.SoundFormatChanged = false;
@@ -812,19 +1095,22 @@ static void Emulate(EmulateSpecStruct* espec)
     memmove(espec->SoundBuf, espec->SoundBuf + expected_delta * 2, espec->SoundBufSize * 2 * sizeof(int16));
    }
   }
-
+  //
+  MDFN_MidSync(espec, MIDSYNC_FLAG_NONE);
+  PPU_SyncMT();
+  //
   MDFNSS_LoadSM(SpecExSS, true);
   SpecExSS->rewind();
  } 
 
- if(spc_reader || snsf_loader)
+ if(MDFN_UNLIKELY(spc_reader || snsf_loader))
  {
   espec->LineWidths[0] = ~0;
   Player_Draw(espec->surface, &espec->DisplayRect, 0, espec->SoundBuf, espec->SoundBufSize);
  }
 }
 
-static void DoSimpleCommand(int cmd)
+static MDFN_COLD void DoSimpleCommand(int cmd)
 {
  switch(cmd)
  {
@@ -841,9 +1127,11 @@ static void StateAction(StateMem *sm, const unsigned load, const bool data_only)
   return;
  }
 
+ bool MemSelect = (CPUM.MemSelectCycles == MEMCYC_FAST);
+
  SFORMAT StateRegs[] =
  {
-  SFVAR(MemSelect),
+  SFCONDVAR(load && load <= 0x00102399, MemSelect),
   SFVAR(WRAM),
 
   SFVAR(Multiplicand),
@@ -858,20 +1146,77 @@ static void StateAction(StateMem *sm, const unsigned load, const bool data_only)
 
  MDFNSS_StateAction(sm, load, data_only, StateRegs, "SNES");
 
- CPU_StateAction(sm, load, data_only);
+ if(load && load <= 0x00102399)
+  CPUM.MemSelectCycles = (MemSelect ? MEMCYC_FAST : MEMCYC_SLOW);
+ //
+ //
+ CPU_StateAction(sm, load, data_only, "CPU", "CPUCORE");
  DMA_StateAction(sm, load, data_only);
  APU_StateAction(sm, load, data_only);
  PPU_StateAction(sm, load, data_only);
  CART_StateAction(sm, load, data_only);
+ MSU1_StateAction(sm, load, data_only);
  INPUT_StateAction(sm, load, data_only);
 
-
  if(load)
  {
   ForceEventUpdates(CPUM.timestamp);
  }
 }
 
+static void SetInput(unsigned port, const char *type, uint8* data)
+{
+ if(spc_reader)
+  return;
+
+ INPUT_Set(port, type, data);
+}
+
+MDFN_COLD uint8* GetNV(uint32* size)
+{
+ *size = CART_GetRAMSize();
+
+ return CART_GetRAMPointer();
+}
+
+uint32 SNES_GetRegister(const unsigned int id, char* special, const uint32 special_len)
+{
+ uint32 ret = 0xDEADBEEF;
+
+ switch(id)
+ {
+  case SNES_GSREG_MEMSEL:
+	ret = (CPUM.MemSelectCycles == MEMCYC_FAST);
+	break;
+
+  case SNES_GSREG_TS:
+	ret = CPUM.timestamp;
+	break;
+ }
+
+ return ret;
+}
+
+void SNES_SetRegister(const unsigned int id, uint32 value)
+{
+ switch(id)
+ {
+  case SNES_GSREG_MEMSEL:
+	CPUM.MemSelectCycles = (value & 0x1) ? MEMCYC_FAST : MEMCYC_SLOW;
+	break;
+ }
+}
+
+uint8 PeekWRAM(uint32 addr)
+{
+ return WRAM[addr & 0x1FFFF];
+}
+
+void PokeWRAM(uint32 addr, uint8 val)
+{
+ WRAM[addr & 0x1FFFF] = val;
+}
+
 static const FileExtensionSpecStruct KnownExtensions[] =
 {
  { ".smc", 0, "Super Magicom ROM Image" },
@@ -882,8 +1227,69 @@ static const FileExtensionSpecStruct KnownExtensions[] =
  { NULL, 0, NULL }
 };
 
+static const MDFNSetting_EnumList HFilter_List[] =
+{
+ { "none", PPU_HFILTER_NONE, gettext_noop("None") },
+ { "512", PPU_HFILTER_512, gettext_noop("Force 512."), gettext_noop("Double width of line if it's 256.") },
+ { "phr256blend", PPU_HFILTER_PHR256BLEND, gettext_noop("Pseudo-hires halve-blend."), gettext_noop("Blend line down to 256 pixels if it's pseudo-hires.") },
+ { "phr256blend_auto512", PPU_HFILTER_PHR256BLEND_AUTO512, gettext_noop("Pseudo-hires halve-blend and force 512 if necessary."), gettext_noop("Blend line down to 256 pixels if it's pseudo-hires.  After, double width of line if any other lines use hires mode.") },
+ { "phr256blend_512", PPU_HFILTER_PHR256BLEND_512, gettext_noop("Pseudo-hires halve-blend and force 512."), gettext_noop("Blend line down to 256 pixels if it's pseudo-hires.  After, double width of line if it's 256.") },
+ { "512_blend", PPU_HFILTER_512_BLEND, gettext_noop("Force 512 and blend."), gettext_noop("Double width of line if it's 256.  After, blend line.") },
+
+ { NULL, 0 },
+};
+
+static const MDFNSetting_EnumList Region_List[] =
+{
+ { "auto", REGION_AUTO, gettext_noop("Auto") },
+
+ { "ntsc", REGION_NTSC, gettext_noop("NTSC(North America/Japan)") },
+ { "pal", REGION_PAL, gettext_noop("PAL(Europe)") },
+
+ { "ntsc_lie_auto", REGION_NTSC_LIE_AUTO, gettext_noop("NTSC, but PPU bit as if \"auto\".") },
+ { "pal_lie_auto", REGION_PAL_LIE_AUTO, gettext_noop("PAL, but PPU bit as if \"auto\".") },
+
+ { "ntsc_lie_pal", REGION_NTSC_LIE_PAL, gettext_noop("NTSC, but PPU bit as if PAL.") },
+ { "pal_lie_ntsc", REGION_PAL_LIE_NTSC, gettext_noop("PAL, but PPU bit as if NTSC.") },
+
+ { NULL, 0 }
+};
+
+static const MDFNSetting_EnumList CAspect_List[] =
+{
+ { "0", PPU_CASPECT_DISABLED, gettext_noop("Disabled") },
+ { "1", PPU_CASPECT_ENABLED, gettext_noop("Enabled") },
+
+ { "force_ntsc", PPU_CASPECT_FORCE_NTSC, gettext_noop("Enabled, force NTSC PAR.") },
+ { "force_pal", PPU_CASPECT_FORCE_PAL, gettext_noop("Enabled, force PAL PAR.") },
+ //
+ //
+ //
+ { "disabled", PPU_CASPECT_DISABLED },
+ { "enabled", PPU_CASPECT_ENABLED },
+
+ { NULL, 0 }
+};
+
+static const MDFNSetting_EnumList Renderer_List[] =
+{
+ { "st", PPU_RENDERER_ST, gettext_noop("Single-threaded"), gettext_noop("PPU rendering is performed in the main emulation thread.") },
+ { "mt", PPU_RENDERER_MT, gettext_noop("Multi-threaded"), gettext_noop("PPU rendering is performed in a dedicated thread.") },
+
+ { NULL, 0 }
+};
+
 static const MDFNSetting Settings[] =
 {
+ { "snes_faust.renderer", MDFNSF_NOFLAGS, gettext_noop("PPU renderer."), gettext_noop("If you have only one CPU with one physical CPU core, select the single-threaded renderer for better performance."), MDFNST_ENUM, "st", NULL, NULL, NULL, NULL, Renderer_List },
+
+ { "snes_faust.affinity.ppu", MDFNSF_NOFLAGS, gettext_noop("PPU rendering thread CPU affinity mask."), gettext_noop("Set to 0 to disable changing affinity."), MDFNST_UINT, "0", "0x0000000000000000", "0xFFFFFFFFFFFFFFFF" },
+ { "snes_faust.affinity.msu1.audio", MDFNSF_NOFLAGS, gettext_noop("MSU1 audio read thread CPU affinity mask."), gettext_noop("Set to 0 to disable changing affinity."), MDFNST_UINT, "0", "0x0000000000000000", "0xFFFFFFFFFFFFFFFF" },
+ { "snes_faust.affinity.msu1.data", MDFNSF_NOFLAGS, gettext_noop("MSU1 data read thread CPU affinity mask."), gettext_noop("Set to 0 to disable changing affinity."), MDFNST_UINT, "0", "0x0000000000000000", "0xFFFFFFFFFFFFFFFF" },
+
+ { "snes_faust.frame_begin_vblank", MDFNSF_EMU_STATE | MDFNSF_UNTRUSTED_SAFE | MDFNSF_SUPPRESS_DOC | MDFNSF_NONPERSISTENT, gettext_noop("Begin frame in emulated VBlank."), gettext_noop("Disabling will make the multithreaded PPU renderer more effective, but will also increase latency."), MDFNST_BOOL, "1" },
+
+ { "snes_faust.msu1.resamp_quality", MDFNSF_NOFLAGS, gettext_noop("MSU1 sound quality."), gettext_noop("Higher values correspond to better SNR and better preservation of higher frequencies(\"brightness\"), at the cost of increased computational complexity and a negligible increase in latency.\n\nHigher values will also slightly increase the probability of sample clipping(relevant if Mednafen's volume control settings are set too high), due to increased (time-domain) ringing."), MDFNST_INT, "4", "0", "5" },
  { "snes_faust.resamp_quality", MDFNSF_NOFLAGS, gettext_noop("Sound quality."), gettext_noop("Higher values correspond to better SNR and better preservation of higher frequencies(\"brightness\"), at the cost of increased computational complexity and a negligible increase in latency.\n\nHigher values will also slightly increase the probability of sample clipping(relevant if Mednafen's volume control settings are set too high), due to increased (time-domain) ringing."), MDFNST_INT, "3", "0", "5" },
  { "snes_faust.resamp_rate_error", MDFNSF_NOFLAGS, gettext_noop("Sound output rate tolerance."), gettext_noop("Lower values correspond to better matching of the output rate of the resampler to the actual desired output rate, at the expense of increased RAM usage and poorer CPU cache utilization."), MDFNST_FLOAT, "0.000035", "0.0000001", "0.0015" },
 
@@ -893,20 +1299,34 @@ static const MDFNSetting Settings[] =
  { "snes_faust.input.sport1.multitap", MDFNSF_EMU_STATE | MDFNSF_UNTRUSTED_SAFE, gettext_noop("Enable multitap on SNES port 1."), NULL, MDFNST_BOOL, "0" },
  { "snes_faust.input.sport2.multitap", MDFNSF_EMU_STATE | MDFNSF_UNTRUSTED_SAFE, gettext_noop("Enable multitap on SNES port 2."), NULL, MDFNST_BOOL, "0" },
 
- { "snes_faust.correct_aspect", MDFNSF_NOFLAGS, gettext_noop("Correct aspect ratio."), NULL, MDFNST_BOOL, "1" },
+ { "snes_faust.region", MDFNSF_EMU_STATE | MDFNSF_UNTRUSTED_SAFE, gettext_noop("Region of SNES to emulate."), NULL, MDFNST_ENUM, "auto", NULL, NULL, NULL, NULL, Region_List },
+
+ { "snes_faust.correct_aspect", MDFNSF_NOFLAGS, gettext_noop("Correct aspect ratio."), NULL, MDFNST_ENUM, "1", NULL, NULL, NULL, NULL, CAspect_List },
+
+ { "snes_faust.slstart", MDFNSF_NOFLAGS, gettext_noop("First displayed scanline in NTSC mode."), NULL, MDFNST_INT, "0", "0", "223" },
+ { "snes_faust.slend", MDFNSF_NOFLAGS, gettext_noop("Last displayed scanline in NTSC mode."), NULL, MDFNST_INT, "223", "0", "223" },
+
+ { "snes_faust.slstartp", MDFNSF_NOFLAGS, gettext_noop("First displayed scanline in PAL mode."), NULL, MDFNST_INT, "0", "0", "238" },
+ { "snes_faust.slendp", MDFNSF_NOFLAGS, gettext_noop("Last displayed scanline in PAL mode."), NULL, MDFNST_INT, "238", "0", "238" },
+
+ { "snes_faust.h_filter", MDFNSF_NOFLAGS, gettext_noop("Horizontal blending/doubling filter."), NULL, MDFNST_ENUM, "none", NULL, NULL, NULL, NULL, HFilter_List },
+
+ { "snes_faust.cx4.clock_rate", MDFNSF_EMU_STATE | MDFNSF_UNTRUSTED_SAFE, gettext_noop("CX4 clock rate, specified in percentage of normal."), gettext_noop("Overclocking the CX4 will cause or worsen attract mode desynchronization."), MDFNST_UINT, "100", "100", "500" },
+ { "snes_faust.superfx.clock_rate", MDFNSF_EMU_STATE | MDFNSF_UNTRUSTED_SAFE, gettext_noop("Super FX clock rate, specified in percentage of normal."), gettext_noop("Overclocking the Super FX will cause or worsen attract mode desynchronization."), MDFNST_UINT, "100", "25", "500" },
+ { "snes_faust.superfx.icache", MDFNSF_EMU_STATE | MDFNSF_UNTRUSTED_SAFE, gettext_noop("Enable SuperFX instruction cache emulation."), gettext_noop("Enabling will likely increase CPU usage."), MDFNST_BOOL, "0" },
 
  { NULL }
 };
 
 static const CheatInfoStruct CheatInfo =
 {
- NULL,
- NULL,
+ CheatInstallReadPatch,
+ CheatRemoveReadPatches,
 
  CheatMemRead,
  CheatMemWrite,
 
- CheatFormatInfo_Empty,
+ CheatFormats_SNES,
  false
 };
 
@@ -920,12 +1340,13 @@ MDFNGI EmulatedSNES_Faust =
  "Super Nintendo Entertainment System/Super Famicom",
  KnownExtensions,
  MODPRIO_INTERNAL_LOW,
- //#ifdef WANT_DEBUGGER
- //&SNES_DBGInfo,
- //#else
+ #ifdef SNES_DBG_ENABLE
+ &DBG_DBGInfo,
+ #else
  NULL,
- //#endif
+ #endif
  INPUT_PortInfo,
+ NULL,
  Load,
  TestMagic,
  NULL,
@@ -947,9 +1368,13 @@ MDFNGI EmulatedSNES_Faust =
  StateAction,
  Emulate,
  NULL,
- INPUT_Set,
+ SetInput,
  NULL,
  DoSimpleCommand,
+/*
+ Reset,
+ GetNV,
+*/
  NULL,
  Settings,
  0,
