@@ -2,7 +2,7 @@
 /* Mednafen Sega Saturn Emulation Module                                      */
 /******************************************************************************/
 /* vdp1.cpp - VDP1 Emulation
-**  Copyright (C) 2015-2019 Mednafen Team
+**  Copyright (C) 2015-2020 Mednafen Team
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -19,14 +19,17 @@
 ** 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
+// TODO: Draw timing for small lines(somewhere between 2 and 15 pixels wide) on the VDP1 seems weird; investigate further
+// before making timing changes to the drawing code.
+
+// TODO: Draw timing for (large) primitives should be about 20% higher.
+
+// TODO: Draw timing for shrunken(even just slightly) sprite lines with HSS disabled should be 100% higher.
+
 // TODO: 32-bit writes from the SH-2 CPUs to VDP1 registers seem to be broken on a Saturn; test, and implement here.
 
-// TODO: COPR shouldn't be updated(from the view of the program) until the current command finishes.
-
 // TODO: Check to see what registers are reset on reset.
 
-// TODO: Fix preclipping when raw system clipping values have bit12==1(sign bit?).
-
 // TODO: SS_SetPhysMemMap(0x05C80000, 0x05CFFFFF, FB[FBDrawWhich], sizeof(FB[0]));
 //  (...but goes weird in 8bpp rotated mode...)
 
@@ -51,7 +54,74 @@ namespace MDFN_IEN_SS
 namespace VDP1
 {
 
-#if 0
+uint8 spr_w_shift_tab[8];
+uint8 gouraud_lut[0x40];
+line_data LineData;
+line_inner_data LineInnerData;
+prim_data PrimData;
+
+int32 SysClipX, SysClipY;
+int32 UserClipX0, UserClipY0, UserClipX1, UserClipY1;
+
+int32 LocalX, LocalY;
+
+uint8 TVMR;
+uint8 FBCR;
+static uint8 PTMR;
+static uint8 EDSR;
+
+uint16* FBDrawWhichPtr;
+static bool FBDrawWhich;
+
+static bool DrawingActive;
+static uint32 CurCommandAddr;
+static int32 RetCommandAddr;
+static uint16 LOPR;
+
+static sscpu_timestamp_t lastts;
+static int32 CycleCounter;
+static int32 CommandPhase;
+static uint16 CommandData[0x10];
+uint32 DTACounter;
+
+static bool vb_status, hb_status;
+static bool vbcdpending;
+static bool FBManualPending;
+static bool FBVBErasePending;
+static bool FBVBEraseActive;
+static sscpu_timestamp_t FBVBEraseLastTS;
+static sscpu_timestamp_t LastRWTS;
+
+static uint16 EWDR;	// Erase/Write Data
+static uint16 EWLR;	// Erase/Write Upper Left Coordinate
+static uint16 EWRR;	// Erase/Write Lower Right Coordinate
+
+static struct
+{
+ bool rot8;
+ uint32 fb_x_mask;
+
+ uint32 y_start;
+ uint32 x_start;
+
+ uint32 y_end;
+ uint32 x_bound;
+
+ uint16 fill_data;
+} EraseParams;
+
+static uint32 EraseYCounter;
+
+#if 1
+static uint32 InstantDrawSanityLimit; // ss_horrible_hacks
+#endif
+
+uint16 VRAM[0x40000];
+uint16 FB[2][0x20000];
+//
+//
+//
+#ifdef MDFN_ENABLE_DEV_BUILD
 struct VRAMUsageInfo
 {
  uint64 earliest;
@@ -75,46 +145,58 @@ static void VRAMUsageAddBaseTime(int32 amount)
 
 static void VRAMUsageStart(void)
 {
- for(size_t i = 0; i < 0x40000; i++)
+ if(MDFN_UNLIKELY(ss_dbg_mask & SS_DBG_VDP1_RACE))
  {
-  VRAMDrawReads[i].earliest = ~(uint64)0;
-  VRAMDrawReads[i].latest = ~(uint64)0;
+  for(size_t i = 0; i < 0x40000; i++)
+  {
+   VRAMDrawReads[i].earliest = ~(uint64)0;
+   VRAMDrawReads[i].latest = ~(uint64)0;
 
-  VRAMWrites[i].earliest = ~(uint64)0;
-  VRAMWrites[i].latest = ~(uint64)0;
+   VRAMWrites[i].earliest = ~(uint64)0;
+   VRAMWrites[i].latest = ~(uint64)0;
+  }
  }
  VRAMUsageStartTS = VRAMUsageTSBase;
 }
 
 static void VRAMUsageWrite(uint32 A)
 {
- const uint64 fullts = VRAMUsageTSBase + SH7095_mem_timestamp;
- const size_t index = (A >> 1) & 0x3FFFF;
+ if(MDFN_UNLIKELY(ss_dbg_mask & SS_DBG_VDP1_RACE))
+ {
+  const uint64 fullts = VRAMUsageTSBase + SH7095_mem_timestamp;
+  const size_t index = A & 0x3FFFF;
 
- if(VRAMWrites[index].earliest == ~(uint64)0)
-  VRAMWrites[index].earliest = fullts;
+  if(VRAMWrites[index].earliest == ~(uint64)0)
+   VRAMWrites[index].earliest = fullts;
 
- VRAMWrites[index].latest = fullts;
+  VRAMWrites[index].latest = fullts;
+ }
 }
 
 static void VRAMUsageDrawRead(uint32 A)
 {
- const uint64 fullts = VRAMUsageTSBase + SH7095_mem_timestamp;
- const size_t index = (A >> 1) & 0x3FFFF;
+ if(MDFN_UNLIKELY(ss_dbg_mask & SS_DBG_VDP1_RACE))
+ {
+  const uint64 fullts = VRAMUsageTSBase + SH7095_mem_timestamp;
+  const size_t index = A & 0x3FFFF;
 
- if(VRAMDrawReads[index].earliest == ~(uint64)0)
-  VRAMDrawReads[index].earliest = fullts;
+  if(VRAMDrawReads[index].earliest == ~(uint64)0)
+   VRAMDrawReads[index].earliest = fullts;
 
- VRAMDrawReads[index].latest = fullts;
+  VRAMDrawReads[index].latest = fullts;
+ }
 }
 
 static void VRAMUsageEnd(void)
 {
- for(unsigned i = 0; i < 0x40000; i++)
+ if(MDFN_UNLIKELY(ss_dbg_mask & SS_DBG_VDP1_RACE))
  {
-  if(VRAMWrites[i].latest != ~(uint64)0 && VRAMDrawReads[i].latest != ~(uint64)0)
+  for(unsigned i = 0; i < 0x40000; i++)
   {
-   printf("VRAM[0x%06x] Race --- Write, earliest=%llu, latest=%llu --- Draw Read, earliest=%llu, latest=%llu\n", i * 2, (unsigned long long)(VRAMWrites[i].earliest - VRAMUsageStartTS), (unsigned long long)(VRAMWrites[i].latest - VRAMUsageStartTS), (unsigned long long)(VRAMDrawReads[i].earliest - VRAMUsageStartTS), (unsigned long long)(VRAMDrawReads[i].latest - VRAMUsageStartTS));
+   if(VRAMWrites[i].latest != ~(uint64)0 && VRAMDrawReads[i].latest != ~(uint64)0)
+   {
+    SS_DBG(SS_DBG_VDP1_RACE, "[VDP1] VRAM[0x%06x] Race --- Write, earliest=%llu, latest=%llu --- Draw Read, earliest=%llu, latest=%llu\n", i * 2, (unsigned long long)(VRAMWrites[i].earliest - VRAMUsageStartTS), (unsigned long long)(VRAMWrites[i].latest - VRAMUsageStartTS), (unsigned long long)(VRAMDrawReads[i].earliest - VRAMUsageStartTS), (unsigned long long)(VRAMDrawReads[i].latest - VRAMUsageStartTS));
+   }
   }
  }
 }
@@ -126,67 +208,9 @@ static INLINE void VRAMUsageWrite(uint32 A) { }
 static INLINE void VRAMUsageDrawRead(uint32 A) { }
 static INLINE void VRAMUsageEnd(void) { }
 #endif
-
-uint8 spr_w_shift_tab[8];
-line_data LineSetup;
-uint8 gouraud_lut[0x40];
-
-uint16 VRAM[0x40000];
-uint16 FB[2][0x20000];
-bool FBDrawWhich;
-
-static bool FBManualPending;
-
-static bool FBVBErasePending;
-static bool FBVBEraseActive;
-static sscpu_timestamp_t FBVBEraseLastTS;
-static sscpu_timestamp_t LastRWTS; // ss_horrible_hacks
-
-int32 SysClipX, SysClipY;
-int32 UserClipX0, UserClipY0, UserClipX1, UserClipY1;
-int32 LocalX, LocalY;
-
-static uint32 CurCommandAddr;
-static int32 RetCommandAddr;
-static bool DrawingActive;
-
-static uint16 LOPR;
-
-static uint16 EWDR;	// Erase/Write Data
-static uint16 EWLR;	// Erase/Write Upper Left Coordinate
-static uint16 EWRR;	// Erase/Write Lower Right Coordinate
-
-static struct
-{
- bool rot8;
- uint32 fb_x_mask;
-
- uint32 y_start;
- uint32 x_start;
-
- uint32 y_end;
- uint32 x_bound;
-
- uint16 fill_data;
-} EraseParams;
-
-static uint32 EraseYCounter;
-
-uint8 TVMR;
-uint8 FBCR;
-static uint8 PTMR;
-static uint8 EDSR;
-
-static bool vb_status, hb_status;
-static sscpu_timestamp_t lastts;
-static int32 CycleCounter;
-
-#if 1
-static uint32 InstantDrawSanityLimit; // ss_horrible_hacks
-#endif
-
-static bool vbcdpending;
-
+//
+//
+//
 void Init(void)
 {
  vbcdpending = false;
@@ -243,7 +267,9 @@ void Reset(bool powering_up)
    for(unsigned i = 0; i < 0x20000; i++)
     FB[fb][i] = 0xFFFF;
 
-  memset(&LineSetup, 0, sizeof(LineSetup));
+  memset(&LineData, 0, sizeof(LineData));
+  memset(&LineInnerData, 0, sizeof(LineInnerData));
+  memset(&PrimData, 0, sizeof(PrimData));
 
   //
   // Registers with somewhat undefined state on power-on:
@@ -265,6 +291,7 @@ void Reset(bool powering_up)
  }
 
  FBDrawWhich = 0;
+ FBDrawWhichPtr = FB[FBDrawWhich];
  //SS_SetPhysMemMap(0x05C80000, 0x05CFFFFF, FB[FBDrawWhich], sizeof(FB[0]), true);
 
  FBManualPending = false;
@@ -275,6 +302,11 @@ void Reset(bool powering_up)
  CurCommandAddr = 0;
  RetCommandAddr = -1;
  DrawingActive = false;
+ CycleCounter = 0;
+ CommandPhase = 0;
+ memset(CommandData, 0, sizeof(CommandData));
+ InstantDrawSanityLimit = 0;
+ DTACounter = 0;
 
  //
  // Begin registers/variables confirmed to be initialized on reset.
@@ -287,26 +319,23 @@ void Reset(bool powering_up)
 
  memset(&EraseParams, 0, sizeof(EraseParams));
  EraseYCounter = ~0U;
-
- CycleCounter = 0;
- InstantDrawSanityLimit = 0;
 }
 
 static int32 CMD_SetUserClip(const uint16* cmd_data)
 {
- UserClipX0 = cmd_data[0x6] & 0x3FF;
- UserClipY0 = cmd_data[0x7] & 0x1FF;
+ UserClipX0 = cmd_data[0x6] & 0x1FFF;
+ UserClipY0 = cmd_data[0x7] & 0x1FFF;
 
- UserClipX1 = cmd_data[0xA] & 0x3FF;
- UserClipY1 = cmd_data[0xB] & 0x1FF;
+ UserClipX1 = cmd_data[0xA] & 0x1FFF;
+ UserClipY1 = cmd_data[0xB] & 0x1FFF;
 
  return 0;
 }
 
 int32 CMD_SetSystemClip(const uint16* cmd_data)
 {
- SysClipX = cmd_data[0xA] & 0x3FF;
- SysClipY = cmd_data[0xB] & 0x1FF;
+ SysClipX = cmd_data[0xA] & 0x1FFF;
+ SysClipY = cmd_data[0xB] & 0x1FFF;
 
  return 0;
 }
@@ -322,7 +351,7 @@ int32 CMD_SetLocalCoord(const uint16* cmd_data)
 template<unsigned ECDSPDMode>
 static uint32 MDFN_FASTCALL TexFetch(uint32 x)
 {
- const uint32 base = LineSetup.tex_base;
+ const uint32 base = LineData.tex_base;
  const bool ECD = ECDSPDMode & 0x10;
  const bool SPD = ECDSPDMode & 0x08;
  const unsigned ColorMode = ECDSPDMode & 0x07;
@@ -338,10 +367,10 @@ static uint32 MDFN_FASTCALL TexFetch(uint32 x)
 
 	if(!ECD && rtd == 0xF)
 	{
-	 LineSetup.ec_count--;	
+	 LineData.ec_count--;	
 	 return -1;
 	}
-	ret_or = LineSetup.cb_or;
+	ret_or = LineData.cb_or;
 	
 	if(!SPD) ret_or |= (int32)(rtd - 1) >> 31;
 
@@ -353,13 +382,13 @@ static uint32 MDFN_FASTCALL TexFetch(uint32 x)
 
 	if(!ECD && rtd == 0xF)
 	{
-	 LineSetup.ec_count--;
+	 LineData.ec_count--;
 	 return -1;
 	}
 
 	if(!SPD) ret_or |= (int32)(rtd - 1) >> 31;
 
-	return LineSetup.CLUT[rtd] | ret_or;
+	return LineData.CLUT[rtd] | ret_or;
 
   case 2:	// 64 colors, color bank
 	rtd = (VRAM[(base + (x >> 1)) & 0x3FFFF] >> (((x & 0x1) ^ 0x1) << 3)) & 0xFF;
@@ -367,11 +396,11 @@ static uint32 MDFN_FASTCALL TexFetch(uint32 x)
 
 	if(!ECD && rtd == 0xFF)
 	{
-	 LineSetup.ec_count--;
+	 LineData.ec_count--;
 	 return -1;
 	}
 
-	ret_or = LineSetup.cb_or;
+	ret_or = LineData.cb_or;
 
 	if(!SPD) ret_or |= (int32)(rtd - 1) >> 31;
 
@@ -383,11 +412,11 @@ static uint32 MDFN_FASTCALL TexFetch(uint32 x)
 
 	if(!ECD && rtd == 0xFF)
 	{
-	 LineSetup.ec_count--;
+	 LineData.ec_count--;
 	 return -1;
 	}
 
-	ret_or = LineSetup.cb_or;
+	ret_or = LineData.cb_or;
 
 	if(!SPD) ret_or |= (int32)(rtd - 1) >> 31;
 
@@ -399,11 +428,11 @@ static uint32 MDFN_FASTCALL TexFetch(uint32 x)
 
 	if(!ECD && rtd == 0xFF)
 	{
-	 LineSetup.ec_count--;
+	 LineData.ec_count--;
 	 return -1;
 	}
 
-	ret_or = LineSetup.cb_or;
+	ret_or = LineData.cb_or;
 
 	if(!SPD) ret_or |= (int32)(rtd - 1) >> 31;
 
@@ -420,7 +449,7 @@ static uint32 MDFN_FASTCALL TexFetch(uint32 x)
 
 	if(!ECD && (rtd & 0xC000) == 0x4000)
 	{
-	 LineSetup.ec_count--;
+	 LineData.ec_count--;
 	 return -1;
 	}
 
@@ -450,6 +479,362 @@ MDFN_HIDE extern uint32 (MDFN_FASTCALL *const TexFetchTab[0x20])(uint32 x) =
  #undef TF
 };
 
+bool SetupDrawLine(int32* const cycle_counter, const bool AA, const bool Textured, const uint16 mode)
+{
+ const bool HSS = (mode & 0x1000);
+ const bool PCD = (mode & 0x800);
+ const bool UserClipEn = (mode & 0x400);
+ const bool UserClipMode = (mode & 0x200);
+ //const bool ECD = (mode & 0x80);
+ //const bool SPD = (mode & 0x40);
+ const bool GouraudEn = (mode & 0x8004) == 0x4;
+ line_vertex p0 = LineData.p[0];
+ line_vertex p1 = LineData.p[1];
+ line_inner_data& lid = LineInnerData;
+ bool clipped = false;
+
+ p0.x &= 0x1FFF;
+ p0.y &= 0x1FFF;
+ p1.x &= 0x1FFF;
+ p1.y &= 0x1FFF;
+
+ //printf("(0x%04x,0x%04x) -> (0x%04x,0x%04x)\n", p0.x, p0.y, p1.x, p1.y);
+
+ if(!PCD)
+ {
+  bool swapped = false;
+
+  *cycle_counter += 4;
+
+  if(UserClipEn && !UserClipMode)
+  {
+   // Ignore system clipping WRT pre-clip for UserClipEn == 1 && UserClipMode == 0
+   clipped |= (((UserClipX1 - p0.x) & (UserClipX1 - p1.x)) | ((p0.x - UserClipX0) & (p1.x - UserClipX0))) & 0x1000;
+   clipped |= (((UserClipY1 - p0.y) & (UserClipY1 - p1.y)) | ((p0.y - UserClipY0) & (p1.y - UserClipY0))) & 0x1000;
+
+   swapped = (p0.y == p1.y) & ((p0.x < UserClipX0) | (p0.x > UserClipX1));
+  }
+  else
+  {
+   clipped |= (((SysClipX - p0.x) & (SysClipX - p1.x)) | (p0.x & p1.x)) & 0x1000;
+   clipped |= (((SysClipY - p0.y) & (SysClipY - p1.y)) | (p0.y & p1.y)) & 0x1000;
+
+   swapped = (p0.y == p1.y) & (p0.x > SysClipX);
+  }
+  //
+  // VDP1 reduces the line into a point to clip it, and it can be seen in the framebuffer under
+  // certain conditions relating to coordinate precision.
+  //
+  if(clipped)
+   p1 = p0;
+  else if(swapped)
+   std::swap(p0, p1);
+ }
+
+ *cycle_counter += 8;
+
+ //
+ //
+ const int32 dx = sign_x_to_s32(13, p1.x - p0.x);
+ const int32 dy = sign_x_to_s32(13, p1.y - p0.y);
+ const int32 abs_dx = abs(dx); // & 0xFFF;
+ const int32 abs_dy = abs(dy); // & 0xFFF;
+ const int32 max_adx_ady = std::max<int32>(abs_dx, abs_dy);
+ const int32 x_inc = (dx >= 0) ? 1 : -1;
+ const int32 y_inc = (dy >= 0) ? 1 : -1;
+
+ lid.x_inc = (x_inc & 0x7FF) <<  0;
+ lid.y_inc = (y_inc & 0x7FF) << 16;
+ lid.xy = (p0.x & 0x7FF) + ((p0.y & 0x7FF) << 16);
+ lid.term_xy = (p1.x & 0x7FF) + ((p1.y & 0x7FF) << 16);
+ lid.drawn_ac = true;	// Drawn all-clipped
+ lid.color = LineData.color;
+
+ //if(max_adx_ady >= 2048)
+ // printf("%d,%d ->  %d, %d\n", p0.x, p0.y, p1.x, p1.y);
+
+ if(GouraudEn)
+  lid.g.Setup(max_adx_ady + 1, p0.g, p1.g);
+
+ if(Textured)
+ {
+  LineData.ec_count = 2;	// Call before tffn()
+
+  if(MDFN_UNLIKELY(max_adx_ady < abs(p1.t - p0.t) && HSS))
+  {
+   LineData.ec_count = 0x7FFFFFFF;
+   lid.t.Setup(max_adx_ady + 1, p0.t >> 1, p1.t >> 1, 2, (bool)(FBCR & FBCR_EOS));
+  }
+  else
+   lid.t.Setup(max_adx_ady + 1, p0.t, p1.t);
+
+  lid.texel = LineData.tffn(lid.t.Current());
+ }
+
+ {
+  int32 aa_x_inc;
+  int32 aa_y_inc;
+
+  if(abs_dy > abs_dx)
+  {
+   if(y_inc < 0)
+   {
+    aa_x_inc =  (x_inc >> 31);
+    aa_y_inc = -(x_inc >> 31);
+   }
+   else
+   {
+    aa_x_inc = -(~x_inc >> 31);
+    aa_y_inc =  (~x_inc >> 31);
+   }
+  }
+  else
+  {
+   if(x_inc < 0)
+   {
+    aa_x_inc = -(~y_inc >> 31);
+    aa_y_inc = -(~y_inc >> 31);
+   }
+   else
+   {
+    aa_x_inc =  (y_inc >> 31);
+    aa_y_inc =  (y_inc >> 31);
+   }
+  }
+  lid.aa_xy_inc = (aa_x_inc & 0x7FF) + ((aa_y_inc & 0x7FF) << 16);
+ }
+
+ // x, y, x_inc, y_inc, aa_x_inc, aa_y_inc, term_x, term_y, error, error_inc, error_adj, t, g, color
+ lid.abs_dy_gt_abs_dx = abs_dy > abs_dx;
+
+ if(abs_dy > abs_dx)
+ {
+  lid.error_inc =  (2 * abs_dx);
+  lid.error_adj = -(2 * abs_dy);
+  lid.error = (abs_dy - (2 * abs_dy)) - 1;
+  lid.error_cmp = 0;
+
+  if(dy < 0 && !AA)
+   lid.error_cmp--;
+
+  lid.error -= lid.error_inc;
+  lid.xy = (lid.xy + (0x8000000 - lid.y_inc)) & 0x07FF07FF;
+ }
+ else
+ {
+  lid.error_inc =  (2 * abs_dy);
+  lid.error_adj = -(2 * abs_dx);
+  lid.error = (abs_dx - (2 * abs_dx)) - 1;
+  lid.error_cmp = 0;
+
+  if(dx < 0 && !AA)
+   lid.error_cmp--;
+
+  lid.error -= lid.error_inc;
+  lid.xy = (lid.xy + (0x800 - lid.x_inc)) & 0x07FF07FF;
+ }
+ if(AA)
+ {
+  lid.error++;
+  lid.error_cmp++;
+ }
+
+ //
+ lid.error_inc <<= 32 - 13;
+ lid.error_adj <<= 32 - 13;
+ lid.error <<= 32 - 13;
+ lid.error_cmp <<= 32 - 13;
+
+ return clipped;
+}
+
+void EdgeStepper::Setup(const bool gourauden, const line_vertex& p0, const line_vertex& p1, const int32 dmax)
+{
+  int32 dx = sign_x_to_s32(13, p1.x - p0.x);
+  int32 dy = sign_x_to_s32(13, p1.y - p0.y);
+  int32 abs_dx = abs(dx);
+  int32 abs_dy = abs(dy);
+  int32 max_adxdy = std::max<int32>(abs_dx, abs_dy);
+
+  //if(abs_dy == abs_dx)
+  // printf("edge abs_dy == abs_dx; dy=%d dx=%d\n", dy, dx);
+
+  //printf("%d %d, %d\n", dx, dy, dmax);
+
+  x = p0.x;
+  x_inc = (dx >= 0) ? 1 : -1;
+  x_error_inc =  (2 * abs_dx);
+  x_error_adj = -(2 * max_adxdy);
+  x_error = (max_adxdy - (2 * max_adxdy)) - 1;
+  x_error_cmp = (dy < 0) ? -1 : 0;
+
+  y = p0.y;
+  y_inc = (dy >= 0) ? 1 : -1;
+  y_error_inc =  (2 * abs_dy);
+  y_error_adj = -(2 * max_adxdy);
+  y_error = (max_adxdy - (2 * max_adxdy)) - 1;
+  y_error_cmp = (dx < 0) ? -1 : 0;
+
+  d_error = dmax - (2 * dmax) - 1;
+  d_error_inc =  (2 * max_adxdy);
+  d_error_adj = -(2 * dmax);
+  d_error_cmp = (((abs_dy > abs_dx) ? dy : dx) < 0) ? -1 : 0;
+  //
+  x_error <<= (32 - 13);
+  x_error_inc <<= (32 - 13);
+  x_error_adj <<= (32 - 13);
+  x_error_cmp <<= (32 - 13);
+
+  y_error <<= (32 - 13);
+  y_error_inc <<= (32 - 13);
+  y_error_adj <<= (32 - 13);
+  y_error_cmp <<= (32 - 13);
+
+  d_error <<= (32 - 13);
+  d_error_inc <<= (32 - 13);
+  d_error_adj <<= (32 - 13);
+  d_error_cmp <<= (32 - 13);
+  //
+  if(gourauden)
+   g.Setup(max_adxdy + 1, p0.g, p1.g);
+}
+
+enum : int { CommandPhaseBias = __COUNTER__ + 1 };
+#define VDP1_EAT_CLOCKS(n)									\
+		{										\
+		 CycleCounter -= (n);								\
+		 case __COUNTER__:								\
+		 if(CycleCounter <= 0)								\
+		 {										\
+		  CommandPhase = __COUNTER__ - CommandPhaseBias - 1;				\
+		  goto Breakout;								\
+		 }										\
+		}										\
+
+
+static INLINE void DoDrawing(void)
+{
+#if 1
+ if(MDFN_UNLIKELY(ss_horrible_hacks & HORRIBLEHACK_VDP1INSTANT))
+  CycleCounter = InstantDrawSanityLimit; 
+#endif
+
+ switch(CommandPhase + CommandPhaseBias)
+ {
+  for(;;)
+  {
+   default:
+   VDP1_EAT_CLOCKS(0);
+
+   // Fetch command data
+   memcpy(CommandData, &VRAM[CurCommandAddr], sizeof(CommandData));
+
+   VDP1_EAT_CLOCKS(16);
+
+   for(unsigned i = 0; i < 16; i++)
+    VRAMUsageDrawRead((CurCommandAddr + i) * 2);
+
+   //SS_DBGTI(SS_DBG_WARNING | SS_DBG_VDP1, "[VDP1] Command @ 0x%06x: 0x%04x\n", CurCommandAddr, cmd_data[0]);
+
+   if(MDFN_LIKELY(!(CommandData[0] & 0xC000)))
+   {
+    if(MDFN_UNLIKELY((CommandData[0] & 0xF) >= 0xC))
+    {
+     DrawingActive = false;
+     VRAMUsageEnd();
+     goto Breakout;
+    }
+    else
+    {
+     static int32 (*const command_table[0xC])(const uint16* cmd_data) =
+     {
+      /* 0x0 */         /* 0x1 */           /* 0x2 */            /* 0x3 */
+      CMD_NormalSprite, CMD_ScaledSprite,   CMD_DistortedSprite, CMD_DistortedSprite,
+
+      /* 0x4 */         /* 0x5 (polyline) *//* 0x6 */            /* 0x7 (polyline) */
+      CMD_Polygon,      CMD_Line,	    CMD_Line,            CMD_Line,
+
+      /* 0x8*/          /* 0x9 */           /* 0xA */            /* 0xB */
+      CMD_SetUserClip,  CMD_SetSystemClip,  CMD_SetLocalCoord,   CMD_SetUserClip
+     };
+
+     static int32 (*const resume_table[0x8])(const uint16* cmd_data) =
+     {
+      /* 0x0 */         /* 0x1 */         /* 0x2 */            /* 0x3 */
+      RESUME_Sprite, RESUME_Sprite, RESUME_Sprite, RESUME_Sprite,
+
+      /* 0x4 */    /* 0x5 */     /* 0x6 */ /* 0x7 */
+      RESUME_Polygon, RESUME_Line, RESUME_Line, RESUME_Line,
+     };
+
+     VDP1_EAT_CLOCKS(command_table[CommandData[0] & 0xF](CommandData));
+     if(!(CommandData[0] & 0x8))
+     {
+      for(;;)
+      {
+       int32 cycles;
+
+       cycles = resume_table[CommandData[0] & 0x7](CommandData);
+
+       if(!cycles)
+        break;
+
+       VDP1_EAT_CLOCKS(cycles);
+      }
+     }
+    }
+   }
+   else if(MDFN_UNLIKELY(CommandData[0] & 0x8000))
+   {
+    SS_DBGTI(SS_DBG_VDP1, "[VDP1] Drawing finished at 0x%05x", CurCommandAddr);
+    DrawingActive = false;
+    VRAMUsageEnd();
+
+    EDSR |= 0x2;	// TODO: Does EDSR reflect IRQ out status?
+
+    SCU_SetInt(SCU_INT_VDP1, true);
+    SCU_SetInt(SCU_INT_VDP1, false);
+    goto Breakout;
+   }
+
+   CurCommandAddr = (CurCommandAddr + 0x10) & 0x3FFFF;
+   switch((CommandData[0] >> 12) & 0x3)
+   {
+    case 0:
+	break;
+
+    case 1:
+	CurCommandAddr = (CommandData[1] << 2) &~ 0xF;
+	break;
+
+    case 2:
+	if(RetCommandAddr < 0)
+	 RetCommandAddr = CurCommandAddr;
+
+	CurCommandAddr = (CommandData[1] << 2) &~ 0xF;
+	break;
+
+    case 3:
+	if(RetCommandAddr >= 0)
+	{
+	 CurCommandAddr = RetCommandAddr;
+	 RetCommandAddr = -1;
+	}
+	break;
+   }
+  //
+  //
+  //
+  }
+ }
+ Breakout:;
+
+#if 1
+ if(MDFN_UNLIKELY(ss_horrible_hacks & HORRIBLEHACK_VDP1INSTANT))
+  InstantDrawSanityLimit = CycleCounter;
+#endif
+}
+
 sscpu_timestamp_t Update(sscpu_timestamp_t timestamp)
 {
  if(MDFN_UNLIKELY(timestamp < lastts))
@@ -476,97 +861,7 @@ sscpu_timestamp_t Update(sscpu_timestamp_t timestamp)
   CycleCounter = 0;
  }
  else if(DrawingActive)
- {
-#if 1
-  if(MDFN_UNLIKELY(ss_horrible_hacks & HORRIBLEHACK_VDP1INSTANT))
-   CycleCounter = InstantDrawSanityLimit; 
-#endif
-
-  while(CycleCounter > 0)
-  {
-   uint16 cmd_data[0x10];
-
-   // Fetch command data
-   memcpy(cmd_data, &VRAM[CurCommandAddr], sizeof(cmd_data));
-   CycleCounter -= 16;
-
-   for(unsigned i = 0; i < 16; i++)
-    VRAMUsageDrawRead((CurCommandAddr + i) * 2);
-
-   //SS_DBGTI(SS_DBG_WARNING | SS_DBG_VDP1, "[VDP1] Command @ 0x%06x: 0x%04x\n", CurCommandAddr, cmd_data[0]);
-
-   if(MDFN_LIKELY(!(cmd_data[0] & 0xC000)))
-   {
-    const unsigned cc = cmd_data[0] & 0xF;
-
-    if(MDFN_UNLIKELY(cc >= 0xC))
-    {
-     DrawingActive = false;
-     VRAMUsageEnd();
-     break;
-    }
-    else
-    {
-     static int32 (*const command_table[0xC])(const uint16* cmd_data) =
-     {
-      /* 0x0 */         /* 0x1 */         /* 0x2 */            /* 0x3 */
-      CMD_NormalSprite, CMD_ScaledSprite, CMD_DistortedSprite, CMD_DistortedSprite,
-
-      /* 0x4 */    /* 0x5 */     /* 0x6 */ /* 0x7 */
-      CMD_Polygon, CMD_Polyline, CMD_Line, CMD_Polyline,
-
-      /* 0x8*/         /* 0x9 */           /* 0xA */          /* 0xB */
-      CMD_SetUserClip, CMD_SetSystemClip,  CMD_SetLocalCoord, CMD_SetUserClip
-     };
-
-     CycleCounter -= command_table[cc](cmd_data);
-    }
-   }
-   else if(MDFN_UNLIKELY(cmd_data[0] & 0x8000))
-   {
-    SS_DBGTI(SS_DBG_VDP1, "[VDP1] Drawing finished at 0x%05x", CurCommandAddr);
-    DrawingActive = false;
-    VRAMUsageEnd();
-
-    EDSR |= 0x2;	// TODO: Does EDSR reflect IRQ out status?
-
-    SCU_SetInt(SCU_INT_VDP1, true);
-    SCU_SetInt(SCU_INT_VDP1, false);
-    break;
-   }
-
-   CurCommandAddr = (CurCommandAddr + 0x10) & 0x3FFFF;
-   switch((cmd_data[0] >> 12) & 0x3)
-   {
-    case 0:
-	break;
-
-    case 1:
-	CurCommandAddr = (cmd_data[1] << 2) &~ 0xF;
-	break;
-
-    case 2:
-	if(RetCommandAddr < 0)
-	 RetCommandAddr = CurCommandAddr;
-
-	CurCommandAddr = (cmd_data[1] << 2) &~ 0xF;
-	break;
-
-    case 3:
-	if(RetCommandAddr >= 0)
-	{
-	 CurCommandAddr = RetCommandAddr;
-	 RetCommandAddr = -1;
-	}
-	break;
-   }
-  }
-
-#if 1
-  if(MDFN_UNLIKELY(ss_horrible_hacks & HORRIBLEHACK_VDP1INSTANT))
-   InstantDrawSanityLimit = CycleCounter;
-#endif
- }
+  DoDrawing();
 
  return timestamp + (DrawingActive ? std::max<int32>(VDP1_UpdateTimingGran, 0 - CycleCounter) : VDP1_IdleTimingGran);
 }
@@ -589,6 +884,7 @@ static void StartDrawing(void)
  CurCommandAddr = 0;
  RetCommandAddr = -1;
  DrawingActive = true;
+ CommandPhase = 0;
  VRAMUsageStart();
 
  CycleCounter = VDP1_UpdateTimingGran;
@@ -685,6 +981,7 @@ void SetHBVB(const sscpu_timestamp_t event_timestamp, const bool new_hb_status,
     }
 
     FBDrawWhich = !FBDrawWhich;
+    FBDrawWhichPtr = FB[FBDrawWhich];
 
     SS_DBGTI(SS_DBG_VDP1, "[VDP1] Displayed framebuffer changed to %d.", !FBDrawWhich);
 
@@ -821,8 +1118,7 @@ void AdjustTS(const int32 delta)
  if(FBVBEraseActive)
   FBVBEraseLastTS += delta;
 
- if(LastRWTS >= 0)
-  LastRWTS += delta;
+ LastRWTS = std::max<sscpu_timestamp_t>(-1000000, LastRWTS + delta);
 
  VRAMUsageAddBaseTime(-delta);
 }
@@ -917,7 +1213,7 @@ static INLINE uint16 ReadReg(const unsigned which)
 //
 MDFN_FASTCALL void Write_CheckDrawSlowdown(uint32 A, sscpu_timestamp_t time_thing)
 {
- if(DrawingActive && (ss_horrible_hacks & HORRIBLEHACK_VDP1RWDRAWSLOWDOWN))
+ if(DrawingActive && time_thing > LastRWTS && (ss_horrible_hacks & HORRIBLEHACK_VDP1RWDRAWSLOWDOWN))
  {
   const int32 count = (A & 0x100000) ? 22 : 25;
   const uint32 a = std::min<uint32>(count, time_thing - LastRWTS);
@@ -930,7 +1226,7 @@ MDFN_FASTCALL void Write_CheckDrawSlowdown(uint32 A, sscpu_timestamp_t time_thin
 MDFN_FASTCALL void Read_CheckDrawSlowdown(uint32 A, sscpu_timestamp_t time_thing)
 {
  //printf("%08x\n", A);
- if(!(A & 0x100000) && DrawingActive && (ss_horrible_hacks & HORRIBLEHACK_VDP1RWDRAWSLOWDOWN))
+ if(!(A & 0x100000) && time_thing > LastRWTS && DrawingActive && (ss_horrible_hacks & HORRIBLEHACK_VDP1RWDRAWSLOWDOWN))
  {
   const int32 count = (A & 0x80000) ? 44 : 41;
   const uint32 a = std::min<uint32>(count, time_thing - LastRWTS);
@@ -946,7 +1242,7 @@ MDFN_FASTCALL void Write8_DB(uint32 A, uint16 DB)
 
  if(A < 0x80000)
  {
-  VRAMUsageWrite(A);
+  VRAMUsageWrite(A >> 1);
   SS_DBGTI(SS_DBG_VDP1_VRAMW, "[VDP1] Write to VRAM: 0x%02x->VRAM[0x%05x]", (DB >> (((A & 1) ^ 1) << 3)) & 0xFF, A);
   ne16_wbo_be<uint8>(VRAM, A, DB >> (((A & 1) ^ 1) << 3) );
   return;
@@ -956,7 +1252,7 @@ MDFN_FASTCALL void Write8_DB(uint32 A, uint16 DB)
  {
   uint32 FBA = A;
 
-  SS_DBGTI(SS_DBG_VDP1_FBW, "[VDP1] Write to FB: 0x%02x->FB[%d][0x%05x]", (DB >> (((A & 1) ^ 1) << 3)) & 0xFF, FBDrawWhich, A & 0x3FFFF);
+  SS_DBGTI(SS_DBG_VDP1_FBW, "[VDP1] Write to FB: 0x%02x->FB[%d][0x%05x] CycleCounter=%d", (DB >> (((A & 1) ^ 1) << 3)) & 0xFF, FBDrawWhich, A & 0x3FFFF, CycleCounter);
 
   if((TVMR & (TVMR_8BPP | TVMR_ROTATE)) == (TVMR_8BPP | TVMR_ROTATE))
    FBA = (FBA & 0x1FF) | ((FBA << 1) & 0x3FC00) | ((FBA >> 8) & 0x200);
@@ -975,7 +1271,7 @@ MDFN_FASTCALL void Write16_DB(uint32 A, uint16 DB)
 
  if(A < 0x80000)
  {
-  VRAMUsageWrite(A);
+  VRAMUsageWrite(A >> 1);
   SS_DBGTI(SS_DBG_VDP1_VRAMW, "[VDP1] Write to VRAM: 0x%04x->VRAM[0x%05x]", DB, A);
   VRAM[A >> 1] = DB;
   return;
@@ -985,7 +1281,7 @@ MDFN_FASTCALL void Write16_DB(uint32 A, uint16 DB)
  {
   uint32 FBA = A;
 
-  SS_DBGTI(SS_DBG_VDP1_FBW, "[VDP1] Write to FB: 0x%04x->FB[%d][0x%05x]", DB, FBDrawWhich, A & 0x3FFFF);
+  SS_DBGTI(SS_DBG_VDP1_FBW, "[VDP1] Write to FB: 0x%04x->FB[%d][0x%05x] CycleCounter=%d", DB, FBDrawWhich, A & 0x3FFFF, CycleCounter);
 
   if((TVMR & (TVMR_8BPP | TVMR_ROTATE)) == (TVMR_8BPP | TVMR_ROTATE))
    FBA = (FBA & 0x1FF) | ((FBA << 1) & 0x3FC00) | ((FBA >> 8) & 0x200);
@@ -1019,6 +1315,93 @@ MDFN_FASTCALL uint16 Read16_DB(uint32 A)
 
 void StateAction(StateMem* sm, const unsigned load, const bool data_only)
 {
+ SFORMAT Prim_StateRegs[] =
+ {
+  SFVAR(PrimData.e->d_error, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->d_error_inc, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->d_error_adj, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->d_error_cmp, 0x2, sizeof(*PrimData.e), PrimData.e),
+
+  SFVAR(PrimData.e->x, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->x_inc, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->x_error, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->x_error_inc, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->x_error_adj, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->x_error_cmp, 0x2, sizeof(*PrimData.e), PrimData.e),
+
+  SFVAR(PrimData.e->y, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->y_inc, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->y_error, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->y_error_inc, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->y_error_adj, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->y_error_cmp, 0x2, sizeof(*PrimData.e), PrimData.e),
+
+  SFVAR(PrimData.e->g.g, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->g.intinc, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->g.ginc, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->g.error, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->g.error_inc, 0x2, sizeof(*PrimData.e), PrimData.e),
+  SFVAR(PrimData.e->g.error_adj, 0x2, sizeof(*PrimData.e), PrimData.e),
+
+  SFVAR(PrimData.big_t.t),
+  SFVAR(PrimData.big_t.tinc),
+  SFVAR(PrimData.big_t.error),
+  SFVAR(PrimData.big_t.error_inc),
+  SFVAR(PrimData.big_t.error_adj),
+
+  SFVAR(PrimData.iter),
+  SFVAR(PrimData.tex_base),
+  SFVAR(PrimData.need_line_resume),
+  //
+  //
+  //
+  SFVAR(LineInnerData.xy),
+  SFVAR(LineInnerData.error),
+  SFVAR(LineInnerData.drawn_ac),
+
+  SFVAR(LineInnerData.texel),
+
+  SFVAR(LineInnerData.t.t),
+  SFVAR(LineInnerData.t.tinc),
+  SFVAR(LineInnerData.t.error),
+  SFVAR(LineInnerData.t.error_inc),
+  SFVAR(LineInnerData.t.error_adj),
+
+  SFVAR(LineInnerData.g.g),
+  SFVAR(LineInnerData.g.intinc),
+  SFVAR(LineInnerData.g.ginc),
+  SFVAR(LineInnerData.g.error),
+  SFVAR(LineInnerData.g.error_inc),
+  SFVAR(LineInnerData.g.error_adj),
+
+  SFVAR(LineInnerData.x_inc),
+  SFVAR(LineInnerData.y_inc),
+  SFVAR(LineInnerData.aa_xy_inc),
+  SFVAR(LineInnerData.term_xy),
+
+  SFVAR(LineInnerData.error_cmp),
+  SFVAR(LineInnerData.error_inc),
+  SFVAR(LineInnerData.error_adj),
+
+  SFVAR(LineInnerData.color),
+
+  SFVAR(LineInnerData.abs_dy_gt_abs_dx),
+  //
+  //
+  //
+  SFVAR(LineData.p->t, 0x2, sizeof(*LineData.p), LineData.p),
+  SFVAR(LineData.color),
+  SFVAR(LineData.ec_count),
+  //uint32 (MDFN_FASTCALL *tffn)(uint32);
+  SFVAR(LineData.CLUT),
+  SFVAR(LineData.cb_or),
+  SFVAR(LineData.tex_base),
+  //
+  //
+  //
+  SFEND
+ };
+
  SFORMAT StateRegs[] =
  {
   SFVAR(VRAM),
@@ -1072,6 +1455,9 @@ void StateAction(StateMem* sm, const unsigned load, const bool data_only)
   SFVAR(hb_status),
   SFVAR(lastts),
   SFVAR(CycleCounter),
+  SFVAR(CommandPhase),
+  SFVAR(CommandData),
+  SFVAR(DTACounter),
 
   SFVAR(vbcdpending),
 
@@ -1079,6 +1465,8 @@ void StateAction(StateMem* sm, const unsigned load, const bool data_only)
 
   SFVAR(InstantDrawSanityLimit),
 
+  SFLINK(Prim_StateRegs),
+
   SFEND
  };
 
@@ -1090,6 +1478,8 @@ void StateAction(StateMem* sm, const unsigned load, const bool data_only)
   if(RetCommandAddr >= 0)
    RetCommandAddr &= 0x3FFFF;
 
+  DTACounter &= 0xFF;
+
   EraseParams.fb_x_mask = EraseParams.rot8 ? 0xFF : 0x1FF;
 
   EraseParams.y_start &= 0x1FF;
@@ -1097,6 +1487,13 @@ void StateAction(StateMem* sm, const unsigned load, const bool data_only)
 
   EraseParams.y_end &= 0x1FF;
   EraseParams.x_bound &= 0x7F << 3;
+  //
+  FBDrawWhichPtr = FB[FBDrawWhich];
+
+  if(load < 0x00102500)
+  {
+   CommandPhase = 0;
+  }
  }
 }
 
@@ -1175,6 +1572,62 @@ uint32 GetRegister(const unsigned id, char* const special, const uint32 special_
 void SetRegister(const unsigned id, const uint32 value)
 {
  // TODO
+ switch(id)
+ {
+  case GSREG_SYSCLIPX:
+	SysClipX = value & 0x1FFF;
+	break;
+
+  case GSREG_SYSCLIPY:
+	SysClipY = value & 0x1FFF;
+	break;
+
+  case GSREG_USERCLIPX0:
+	UserClipX0 = value & 0x1FFF;
+	break;
+
+  case GSREG_USERCLIPY0:
+	UserClipY0 = value & 0x1FFF;
+	break;
+
+  case GSREG_USERCLIPX1:
+	UserClipX1 = value & 0x1FFF;
+	break;
+
+  case GSREG_USERCLIPY1:
+	UserClipY1 = value & 0x1FFF;
+	break;
+
+/*
+  case GSREG_LOCALX:
+	ret = LocalX;
+	break;
+
+  case GSREG_LOCALY:
+	ret = LocalY;
+ 	break;
+
+  case GSREG_TVMR:
+	ret = TVMR;
+	break;
+
+  case GSREG_FBCR:
+	ret = FBCR;
+	break;
+
+  case GSREG_EWDR:
+	ret = EWDR;
+	break;
+
+  case GSREG_EWLR:
+	ret = EWLR;
+	break;
+
+  case GSREG_EWRR:
+	ret = EWRR;
+	break;
+*/
+ }
 }
 
 }
