@@ -38,6 +38,7 @@
 
 namespace MDFN_IEN_SS
 {
+#include "sh7095.h"
 
 namespace VDP2
 {
@@ -370,6 +371,16 @@ static INLINE void IncVCounter(const sscpu_timestamp_t event_timestamp)
   Window[0].YEndMet = Window[1].YEndMet = false;
  }
 
+#if 1
+ if(MDFN_UNLIKELY(ss_horrible_hacks & HORRIBLEHACK_NOSH2DMALINE106))
+ {
+  const bool s = (VCounter == (VTimings[PAL][VRes][VPHASE__COUNT - 1] - 1));
+
+  for(size_t i = 0; i < 2; i++)
+   CPU[i].SetExtHaltDMAKludgeFromVDP2(s);
+ }
+#endif
+
  // - 1, so the CPU loop will  have plenty of time to exit before we reach non-hblank top border area
  // (exit granularity could be large if program is executing from SCSP RAM space, for example).
  if(VCounter == (VTimings[PAL][VRes][VPHASE_TOP_BLANKING] - 1))
@@ -873,17 +884,19 @@ void AdjustTS(const int32 delta)
 }
 
 
-void Init(const bool IsPAL)
+void Init(const bool IsPAL, const uint64 affinity)
 {
  SurfInterlaceField = -1;
  PAL = IsPAL;
  lastts = 0;
+ CRTLineCounter = 0x80000000U;
+ Clock28M = false;
 
  SS_SetPhysMemMap(0x05E00000, 0x05EFFFFF, VRAM, 0x80000, true);
 
  ExLatchIn = false;
 
- VDP2REND_Init(IsPAL);
+ VDP2REND_Init(IsPAL, affinity);
 }
 
 void SetGetVideoParams(MDFNGI* gi, const bool caspect, const int sls, const int sle, const bool show_h_overscan, const bool dohblend)
@@ -906,6 +919,8 @@ void Kill(void)
 //
 void Reset(bool powering_up)
 {
+ memset(RawRegs, 0, sizeof(RawRegs));
+
  DisplayOn = false;
  BorderMode = false;
  ExLatchEnable = false;
@@ -924,6 +939,9 @@ void Reset(bool powering_up)
  VCounter = 0;
  Odd = true;
 
+ for(size_t i = 0; i < 2; i++)
+  CPU[i].SetExtHaltDMAKludgeFromVDP2(false);
+
  RAMCTL_Raw = 0;
  CRAM_Mode = 0;
 
