@@ -27,6 +27,34 @@
 #include <mednafen/mednafen.h>
 #include "string.h"
 
+#include <trio/trio.h>
+
+namespace Mednafen
+{
+
+static int AppendSS(void* data, int c)
+{
+ std::string* ss = (std::string*)data;
+
+ ss->push_back(c);
+
+ return 1;
+}
+
+std::string MDFN_sprintf(const char* format, ...)
+{
+ std::string ret;
+ va_list ap;
+
+ ret.reserve(64);
+
+ va_start(ap, format);
+ trio_vcprintf(AppendSS, &ret, format, ap);
+ va_end(ap);
+
+ return ret;
+}
+
 // Remove whitespace from beginning of s
 void MDFN_ltrim(char* s)
 {
@@ -49,24 +77,33 @@ void MDFN_ltrim(char* s)
 }
 
 // Remove whitespace from beginning of s
-void MDFN_ltrim(std::string& s)
+void MDFN_ltrim(std::string* s)
 {
- const size_t len = s.length();
+ const size_t len = s->length();
  size_t di = 0, si = 0;
  bool InWhitespace = true;
 
  while(si < len)
  {
-  if(!InWhitespace || !MDFN_isspace(s[si]))
+  if(!InWhitespace || !MDFN_isspace((*s)[si]))
   {
    InWhitespace = false;
-   s[di] = s[si];
+   (*s)[di] = (*s)[si];
    di++;
   }
   si++;
  }
 
- s.resize(di);
+ s->resize(di);
+}
+
+std::string MDFN_ltrim(const std::string& s)
+{
+ std::string ret = s;
+
+ MDFN_ltrim(&ret);
+
+ return ret;
 }
 
 // Remove whitespace from end of s
@@ -91,9 +128,9 @@ void MDFN_rtrim(char* s)
 }
 
 // Remove whitespace from end of s
-void MDFN_rtrim(std::string& s)
+void MDFN_rtrim(std::string* s)
 {
- const size_t len = s.length();
+ const size_t len = s->length();
 
  if(!len)
   return;
@@ -105,13 +142,22 @@ void MDFN_rtrim(std::string& s)
  {
   x--;
 
-  if(!MDFN_isspace(s[x]))
+  if(!MDFN_isspace((*s)[x]))
    break;
  
   new_len--;
  } while(x);
 
- s.resize(new_len);
+ s->resize(new_len);
+}
+
+std::string MDFN_rtrim(const std::string& s)
+{
+ std::string ret = s;
+
+ MDFN_rtrim(&ret);
+
+ return ret;
 }
 
 void MDFN_trim(char* s)
@@ -120,12 +166,20 @@ void MDFN_trim(char* s)
  MDFN_ltrim(s);
 }
 
-void MDFN_trim(std::string& s)
+void MDFN_trim(std::string* s)
 {
  MDFN_rtrim(s);
  MDFN_ltrim(s);
 }
 
+std::string MDFN_trim(const std::string& s)
+{
+ std::string ret = s;
+
+ MDFN_trim(&ret);
+
+ return ret;
+}
 
 void MDFN_zapctrlchars(char* s)
 {
@@ -141,13 +195,22 @@ void MDFN_zapctrlchars(char* s)
  }
 }
 
-void MDFN_zapctrlchars(std::string& s)
+void MDFN_zapctrlchars(std::string* s)
 {
- for(auto& c : s)
+ for(auto& c : *s)
   if((unsigned char)c < 0x20)
    c = ' ';
 }
 
+std::string MDFN_zapctrlchars(const std::string& s)
+{
+ std::string ret = s;
+
+ MDFN_zapctrlchars(&ret);
+
+ return ret;
+}
+
 void MDFN_strazlower(char* s)
 {
  while(*s)
@@ -157,12 +220,21 @@ void MDFN_strazlower(char* s)
  }
 }
 
-void MDFN_strazlower(std::string& s)
+void MDFN_strazlower(std::string* s)
 {
- for(auto& c : s)
+ for(auto& c : *s)
   c = MDFN_azlower(c);
 }
 
+std::string MDFN_strazlower(const std::string& s)
+{
+ std::string ret = s;
+
+ MDFN_strazlower(&ret);
+
+ return ret;
+}
+
 void MDFN_strazupper(char* s)
 {
  while(*s)
@@ -172,12 +244,21 @@ void MDFN_strazupper(char* s)
  }
 }
 
-void MDFN_strazupper(std::string& s)
+void MDFN_strazupper(std::string* s)
 {
- for(auto& c : s)
+ for(auto& c : *s)
   c = MDFN_azupper(c);
 }
 
+std::string MDFN_strazupper(const std::string& s)
+{
+ std::string ret = s;
+
+ MDFN_strazupper(&ret);
+
+ return ret;
+}
+
 int MDFN_strazicmp(const char* s, const char* t, size_t n)
 {
  if(!n)
@@ -230,6 +311,207 @@ std::vector<std::string> MDFN_strsplit(const std::string& str, const std::string
  return ret;
 }
 
+void MDFN_strunescape(std::string* s)
+{
+ std::string& str = *s;
+ std::string ret;
+ size_t di = 0;
+ bool in_escape = false;
+ unsigned in_octal = 0;
+ unsigned in_hex = 0;
+ unsigned hv = 0;
+
+ for(size_t i = 0; i < str.size(); i++)
+ {
+  if(in_octal)
+  {
+   if(str[i] >= '0' && str[i] <= '7')
+   {
+    hv *= 8;
+    hv += str[i] - '0';
+    in_octal--;
+   }
+   else
+    in_octal = 0;
+
+   if(!in_octal || (i + 1) == str.size())
+    str[di++] = hv;
+  }
+  else if(in_hex)
+  {
+   char lc = MDFN_azlower(str[i]);
+
+   if((lc >= '0' && lc <= '9') || (lc >= 'a' && lc <= 'f'))
+   {
+    hv <<= 4;
+    hv += (lc >= '0' && lc <= '9') ? (lc - '0') : (lc - 'a' + 0xA);
+    in_hex--;
+   }
+   else
+    in_hex = 0;
+
+   if(!in_hex)
+    str[di++] = hv;
+  }
+  else if(in_escape)
+  {
+   if(str[i] >= '0' && str[i] <= '7')
+   {
+    in_octal = 2;
+    hv = str[i] - '0';
+   }
+   else
+   {
+    in_escape = false;
+
+    if(str[i] == 'x')
+    {
+     in_hex = 2;
+     hv = 0;
+    }
+    else if(str[i] == 'o')
+    {
+     in_octal = 3;
+     hv = 0;
+    }
+    else
+    {
+     char c = str[i];
+
+     switch(str[i])
+     {
+      case 'a': c = '\a'; break;
+      case 'b': c = '\b'; break;
+      case 'f': c = '\f'; break;
+      case 'n': c = '\n'; break;
+      case 'r': c = '\r'; break;
+      case 't': c = '\t'; break;
+      case 'v': c = '\v'; break;
+      case '\\': c = '\\'; break;
+      case '\'': c = '\''; break;
+      case '"': c = '"'; break;
+      case '?': c = '?'; break;
+     }
+
+     str[di++] = c;
+    }
+   }
+  }
+  else
+  {
+   if(str[i] == '\\')
+    in_escape = true;
+   else
+    str[di++] = str[i];
+  }
+ }
+
+ assert(di <= str.size());
+ str.resize(di);
+}
+
+std::string MDFN_strunescape(const std::string& str)
+{
+ std::string ret = str;
+
+ MDFN_strunescape(&ret);
+
+ return ret;
+}
+
+std::string MDFN_strescape(const std::string& str)
+{
+ std::string ret;
+ size_t di = 0;
+
+ ret.resize(str.size() * 4);
+
+ for(size_t i = 0; i < str.size(); i++)
+ {
+  unsigned char c = str[i];
+
+  if(c < 0x0E)
+  {
+   static const char tab[0xE] = { '0', '1', '2', '3', '4', '5', '6', 'a', 'b', 't', 'n', 'v', 'f', 'r' };
+
+   ret[di++] = '\\';
+   ret[di++] = tab[c];
+  }
+  else if(c < 0x20 || c == 0x7F)
+  {
+   static const char nybtab[0x10] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
+
+   ret[di++] = '\\';
+   ret[di++] = 'x';
+   ret[di++] = '0' + (c >> 4);
+   ret[di++] = nybtab[c & 0xF]; // (c & 0x0F) + (((c & 0x0F) >= 0x0A) ? 'a' : '0');
+  }
+  else
+  {
+   if(c == '"' || c == '\\')
+    ret[di++] = '\\';
+
+   ret[di++] = c;
+  }
+ }
+
+ ret.resize(di);
+ ret.shrink_to_fit();
+
+ return ret;
+}
+
+std::vector<std::string> MDFN_strargssplit(const std::string& str)
+{
+ std::vector<std::string> ret;
+ std::string tmp;
+ bool tmp_valid = false;
+ bool in_quote = false;
+ bool in_ws = true;
+ int last_c = 0;
+
+ for(size_t i = 0; i < str.size(); i++)
+ {
+  const int new_c = str[i];
+  const bool is_quote = (new_c == '"' && last_c != '\\');
+  const bool new_in_quote = in_quote ^ is_quote;
+  const bool new_in_ws = MDFN_isspace(new_c) & !new_in_quote;
+
+  if(!new_in_ws & !is_quote)
+  {
+   //printf("KA: %c\n", new_c);
+   tmp.push_back(new_c);
+   tmp_valid = true;
+  }
+
+  if((in_quote ^ new_in_quote) & new_in_quote)
+   tmp_valid = true;
+
+  if((in_ws ^ new_in_ws) & new_in_ws)
+  {
+   MDFN_strunescape(&tmp);
+   //printf("borp1: %s\n", tmp.c_str());
+   ret.push_back(tmp);
+   tmp.clear();
+   tmp_valid = false;
+  }
+
+  in_quote = new_in_quote;
+  in_ws = new_in_ws;
+  last_c = new_c;
+ }
+
+ if(tmp_valid)
+ {
+  MDFN_strunescape(&tmp);
+  //printf("borp2: %s\n", tmp.c_str());
+  ret.push_back(tmp);
+ }
+
+ return ret;
+}
+
+
 template<typename T> static void utf_noreplace(T* c) { }
 template<int replacement, typename T> static void utf_replace(T* c) { *c = replacement; }
 static INLINE void dummy_cp_handler(char32_t) { }
@@ -526,9 +808,18 @@ void UTF8_sanitize(size_t len, char* s, bool permit_utf16_surrogates)
  SanitizeUTF8(s, len, permit_utf16_surrogates);
 }
 
-void UTF8_sanitize(std::string& s, bool permit_utf16_surrogates)
+void UTF8_sanitize(std::string* s, bool permit_utf16_surrogates)
 {
- SanitizeUTF8(&s[0], s.size(), permit_utf16_surrogates);
+ SanitizeUTF8(&(*s)[0], s->size(), permit_utf16_surrogates);
+}
+
+std::string UTF8_sanitize(const std::string& s, bool permit_utf16_surrogates)
+{
+ std::string ret = s;
+
+ UTF8_sanitize(&ret, permit_utf16_surrogates);
+
+ return ret;
 }
 
 //
@@ -727,3 +1018,4 @@ std::u16string UTF32_to_UTF16(const char32_t* s, size_t slen, bool* invalid_utf3
  return ret;
 }
 
+}
